/*******************************************************************************
* File Name: CLKGEN1_INT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "CLKGEN1_PVT.h"
#include "cyapicallbacks.h"

/* User code required at start of ISR */
/* `#START CLKGEN1_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: CLKGEN1_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  CLKGEN1_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  CLKGEN1_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  CLKGEN1_txBuffer[CLKGEN1_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(CLKGEN1_TX_ISR)
{
    #if(CLKGEN1_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (CLKGEN1_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef CLKGEN1_TX_ISR_ENTRY_CALLBACK
        CLKGEN1_TX_ISR_EntryCallback();
    #endif /* CLKGEN1_TX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START CLKGEN1_TX_ISR_START` */

    /* `#END` */
    
    #if(CLKGEN1_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(CLKGEN1_txBufferRead != CLKGEN1_txBufferWrite)
        {
            tmpStatus = CLKGEN1_GET_STATUS_TX(CLKGEN1_swStatusTx);
            CLKGEN1_swStatusTx = tmpStatus;

            if(0u != (CLKGEN1_swStatusTx & CLKGEN1_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == CLKGEN1_txBufferFull)
                {
                   CLKGEN1_txBufferRead++;

                    if(CLKGEN1_txBufferRead >= CLKGEN1_TX_BUFFER_SIZE)
                    {
                        CLKGEN1_txBufferRead = 0u;
                    }
                }
                else
                {
                    CLKGEN1_txBufferFull = 0u;
                }

                /* Put data element into the TX FIFO */
                CY_SET_REG8(CLKGEN1_TXDATA_PTR, 
                                             CLKGEN1_txBuffer[CLKGEN1_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(CLKGEN1_txBufferRead == CLKGEN1_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            CLKGEN1_TX_STATUS_MASK_REG &= ((uint8) ~CLKGEN1_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (CLKGEN1_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START CLKGEN1_TX_ISR_END` */

    /* `#END` */
    
    #ifdef CLKGEN1_TX_ISR_EXIT_CALLBACK
        CLKGEN1_TX_ISR_ExitCallback();
    #endif /* CLKGEN1_TX_ISR_EXIT_CALLBACK */
}


/*******************************************************************************
* Function Name: CLKGEN1_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  CLKGEN1_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  CLKGEN1_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  CLKGEN1_rxBuffer[CLKGEN1_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(CLKGEN1_RX_ISR)
{
    #if(CLKGEN1_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (CLKGEN1_RX_SOFTWARE_BUF_ENABLED) */

    #ifdef CLKGEN1_RX_ISR_ENTRY_CALLBACK
        CLKGEN1_RX_ISR_EntryCallback();
    #endif /* CLKGEN1_RX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START CLKGEN1_RX_ISR_START` */

    /* `#END` */
    
    #if(CLKGEN1_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = CLKGEN1_GET_STATUS_RX(CLKGEN1_swStatusRx);
        CLKGEN1_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (CLKGEN1_swStatusRx & CLKGEN1_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG8(CLKGEN1_RXDATA_PTR);

            /* Set next pointer. */
            CLKGEN1_rxBufferWrite++;
            if(CLKGEN1_rxBufferWrite >= CLKGEN1_RX_BUFFER_SIZE)
            {
                CLKGEN1_rxBufferWrite = 0u;
            }

            if(CLKGEN1_rxBufferWrite == CLKGEN1_rxBufferRead)
            {
                CLKGEN1_rxBufferRead++;
                if(CLKGEN1_rxBufferRead >= CLKGEN1_RX_BUFFER_SIZE)
                {
                    CLKGEN1_rxBufferRead = 0u;
                }

                CLKGEN1_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            CLKGEN1_rxBuffer[CLKGEN1_rxBufferWrite] = rxData;

            tmpStatus = CLKGEN1_GET_STATUS_RX(CLKGEN1_swStatusRx);
            CLKGEN1_swStatusRx = tmpStatus;
        }

    #endif /* (CLKGEN1_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START CLKGEN1_RX_ISR_END` */

    /* `#END` */
    
    #ifdef CLKGEN1_RX_ISR_EXIT_CALLBACK
        CLKGEN1_RX_ISR_ExitCallback();
    #endif /* CLKGEN1_RX_ISR_EXIT_CALLBACK */
}

/* [] END OF FILE */
