/*******************************************************************************
* File Name: CLKGEN2_INT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "CLKGEN2_PVT.h"
#include "cyapicallbacks.h"

/* User code required at start of ISR */
/* `#START CLKGEN2_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: CLKGEN2_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  CLKGEN2_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  CLKGEN2_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  CLKGEN2_txBuffer[CLKGEN2_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(CLKGEN2_TX_ISR)
{
    #if(CLKGEN2_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (CLKGEN2_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef CLKGEN2_TX_ISR_ENTRY_CALLBACK
        CLKGEN2_TX_ISR_EntryCallback();
    #endif /* CLKGEN2_TX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START CLKGEN2_TX_ISR_START` */

    /* `#END` */
    
    #if(CLKGEN2_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(CLKGEN2_txBufferRead != CLKGEN2_txBufferWrite)
        {
            tmpStatus = CLKGEN2_GET_STATUS_TX(CLKGEN2_swStatusTx);
            CLKGEN2_swStatusTx = tmpStatus;

            if(0u != (CLKGEN2_swStatusTx & CLKGEN2_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == CLKGEN2_txBufferFull)
                {
                   CLKGEN2_txBufferRead++;

                    if(CLKGEN2_txBufferRead >= CLKGEN2_TX_BUFFER_SIZE)
                    {
                        CLKGEN2_txBufferRead = 0u;
                    }
                }
                else
                {
                    CLKGEN2_txBufferFull = 0u;
                }

                /* Put data element into the TX FIFO */
                CY_SET_REG8(CLKGEN2_TXDATA_PTR, 
                                             CLKGEN2_txBuffer[CLKGEN2_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(CLKGEN2_txBufferRead == CLKGEN2_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            CLKGEN2_TX_STATUS_MASK_REG &= ((uint8) ~CLKGEN2_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (CLKGEN2_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START CLKGEN2_TX_ISR_END` */

    /* `#END` */
    
    #ifdef CLKGEN2_TX_ISR_EXIT_CALLBACK
        CLKGEN2_TX_ISR_ExitCallback();
    #endif /* CLKGEN2_TX_ISR_EXIT_CALLBACK */
}


/*******************************************************************************
* Function Name: CLKGEN2_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  CLKGEN2_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  CLKGEN2_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  CLKGEN2_rxBuffer[CLKGEN2_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(CLKGEN2_RX_ISR)
{
    #if(CLKGEN2_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (CLKGEN2_RX_SOFTWARE_BUF_ENABLED) */

    #ifdef CLKGEN2_RX_ISR_ENTRY_CALLBACK
        CLKGEN2_RX_ISR_EntryCallback();
    #endif /* CLKGEN2_RX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START CLKGEN2_RX_ISR_START` */

    /* `#END` */
    
    #if(CLKGEN2_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = CLKGEN2_GET_STATUS_RX(CLKGEN2_swStatusRx);
        CLKGEN2_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (CLKGEN2_swStatusRx & CLKGEN2_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG8(CLKGEN2_RXDATA_PTR);

            /* Set next pointer. */
            CLKGEN2_rxBufferWrite++;
            if(CLKGEN2_rxBufferWrite >= CLKGEN2_RX_BUFFER_SIZE)
            {
                CLKGEN2_rxBufferWrite = 0u;
            }

            if(CLKGEN2_rxBufferWrite == CLKGEN2_rxBufferRead)
            {
                CLKGEN2_rxBufferRead++;
                if(CLKGEN2_rxBufferRead >= CLKGEN2_RX_BUFFER_SIZE)
                {
                    CLKGEN2_rxBufferRead = 0u;
                }

                CLKGEN2_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            CLKGEN2_rxBuffer[CLKGEN2_rxBufferWrite] = rxData;

            tmpStatus = CLKGEN2_GET_STATUS_RX(CLKGEN2_swStatusRx);
            CLKGEN2_swStatusRx = tmpStatus;
        }

    #endif /* (CLKGEN2_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START CLKGEN2_RX_ISR_END` */

    /* `#END` */
    
    #ifdef CLKGEN2_RX_ISR_EXIT_CALLBACK
        CLKGEN2_RX_ISR_ExitCallback();
    #endif /* CLKGEN2_RX_ISR_EXIT_CALLBACK */
}

/* [] END OF FILE */
