/*******************************************************************************
* File Name: DSPSPI.c
* Version 2.70
*
* Description:
*  This file provides all API functionality of the SPI Slave component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "DSPSPI_PVT.h"

#if (DSPSPI_TX_SOFTWARE_BUF_ENABLED)

    volatile uint8 DSPSPI_txBuffer[DSPSPI_TX_BUFFER_SIZE];
    volatile uint8 DSPSPI_txBufferRead;
    volatile uint8 DSPSPI_txBufferWrite;
    volatile uint8 DSPSPI_txBufferFull;

#endif /* DSPSPI_TX_SOFTWARE_BUF_ENABLED*/

#if (DSPSPI_RX_SOFTWARE_BUF_ENABLED)

    volatile uint8 DSPSPI_rxBuffer[DSPSPI_RX_BUFFER_SIZE];
    volatile uint8 DSPSPI_rxBufferRead;
    volatile uint8 DSPSPI_rxBufferWrite;
    volatile uint8 DSPSPI_rxBufferFull;

#endif /* DSPSPI_RX_SOFTWARE_BUF_ENABLED */

uint8 DSPSPI_initVar = 0u;

volatile uint8 DSPSPI_swStatusTx;
volatile uint8 DSPSPI_swStatusRx;


/*******************************************************************************
* Function Name: DSPSPI_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default SPIS configuration provided with customizer.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  When this function is called it initializes all of the necessary parameters
*  for execution. i.e. setting the initial interrupt mask, configuring the
*  interrupt service routine, configuring the bit-counter parameters and
*  clearing the FIFO and Status Register.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void DSPSPI_Init(void) 
{
    /*Initialize the Bit counter */
    DSPSPI_COUNTER_PERIOD_REG = DSPSPI_BITCTR_INIT;

    /* ISR initialization */
    #if(DSPSPI_INTERNAL_TX_INT_ENABLED)

        CyIntDisable(DSPSPI_TX_ISR_NUMBER);

        /* Set the ISR to point to the DSPSPI_isr Interrupt. */
        (void)CyIntSetVector(DSPSPI_TX_ISR_NUMBER, &DSPSPI_TX_ISR);

        /* Set the priority. */
        CyIntSetPriority(DSPSPI_TX_ISR_NUMBER, DSPSPI_TX_ISR_PRIORITY);

    #endif /* DSPSPI_INTERNAL_TX_INT_ENABLED */

    #if(DSPSPI_INTERNAL_RX_INT_ENABLED)

        CyIntDisable(DSPSPI_RX_ISR_NUMBER);

        /* Set the ISR to point to the DSPSPI_isr Interrupt. */
        (void)CyIntSetVector(DSPSPI_RX_ISR_NUMBER, &DSPSPI_RX_ISR);

        /* Set the priority. */
        CyIntSetPriority(DSPSPI_RX_ISR_NUMBER, DSPSPI_RX_ISR_PRIORITY);

    #endif /* DSPSPI_INTERNAL_RX_INT_ENABLED */

    /* Clear any stray data from the RX and TX FIFO */
    DSPSPI_ClearFIFO();

    #if(DSPSPI_RX_SOFTWARE_BUF_ENABLED)
        DSPSPI_rxBufferFull = 0u;
        DSPSPI_rxBufferRead = 0u;
        DSPSPI_rxBufferWrite = 0u;
    #endif /* DSPSPI_RX_SOFTWARE_BUF_ENABLED */

    #if(DSPSPI_TX_SOFTWARE_BUF_ENABLED)
        DSPSPI_txBufferFull = 0u;
        DSPSPI_txBufferRead = 0u;
        DSPSPI_txBufferWrite = 0u;
    #endif /* DSPSPI_TX_SOFTWARE_BUF_ENABLED */

    (void) DSPSPI_ReadTxStatus(); /* Clear any pending status bits */
    (void) DSPSPI_ReadRxStatus(); /* Clear any pending status bits */


    /* Configure the Initial interrupt mask */
    #if (DSPSPI_TX_SOFTWARE_BUF_ENABLED)
        DSPSPI_TX_STATUS_MASK_REG  = (DSPSPI_TX_INIT_INTERRUPTS_MASK &
                                                (uint8)~DSPSPI_STS_TX_FIFO_NOT_FULL);
    #else /* DSPSPI_TX_SOFTWARE_BUF_ENABLED */
        DSPSPI_TX_STATUS_MASK_REG  = DSPSPI_TX_INIT_INTERRUPTS_MASK;
    #endif /* DSPSPI_TX_SOFTWARE_BUF_ENABLED */

    DSPSPI_RX_STATUS_MASK_REG  = DSPSPI_RX_INIT_INTERRUPTS_MASK;
}


/*******************************************************************************
* Function Name: DSPSPI_Enable
********************************************************************************
*
* Summary:
*  Enable SPIS component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void DSPSPI_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    DSPSPI_COUNTER_CONTROL_REG |= DSPSPI_CNTR_ENABLE;
    DSPSPI_TX_STATUS_ACTL_REG |= DSPSPI_INT_ENABLE;
    DSPSPI_RX_STATUS_ACTL_REG |= DSPSPI_INT_ENABLE;

    CyExitCriticalSection(enableInterrupts);

    #if(DSPSPI_INTERNAL_TX_INT_ENABLED)
        CyIntEnable(DSPSPI_TX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_TX_INT_ENABLED */

    #if(DSPSPI_INTERNAL_RX_INT_ENABLED)
        CyIntEnable(DSPSPI_RX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_RX_INT_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the SPI Slave component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  DSPSPI_initVar - used to check initial configuration, modified on
*  first function call.
*
* Theory:
*  Enable the clock input to enable operation.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void DSPSPI_Start(void) 
{
    if(DSPSPI_initVar == 0u)
    {
        DSPSPI_Init();
        DSPSPI_initVar = 1u;
    }

    DSPSPI_Enable();
}


/*******************************************************************************
* Function Name: DSPSPI_Stop
********************************************************************************
*
* Summary:
*  Disable the SPI Slave component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal interrupt if one is used.
*
*******************************************************************************/
void DSPSPI_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    DSPSPI_TX_STATUS_ACTL_REG &= (uint8)~DSPSPI_INT_ENABLE;
    DSPSPI_RX_STATUS_ACTL_REG &= (uint8)~DSPSPI_INT_ENABLE;

    CyExitCriticalSection(enableInterrupts);

    #if(DSPSPI_INTERNAL_TX_INT_ENABLED)
        CyIntDisable(DSPSPI_TX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_TX_INT_ENABLED */

    #if(DSPSPI_INTERNAL_RX_INT_ENABLED)
        CyIntDisable(DSPSPI_RX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_RX_INT_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_EnableTxInt
********************************************************************************
*
* Summary:
*  Enable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void DSPSPI_EnableTxInt(void) 
{
    #if(DSPSPI_INTERNAL_TX_INT_ENABLED)
        CyIntEnable(DSPSPI_TX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_TX_INT_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_EnableRxInt
********************************************************************************
*
* Summary:
*  Enable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void DSPSPI_EnableRxInt(void) 
{
    #if(DSPSPI_INTERNAL_RX_INT_ENABLED)
        CyIntEnable(DSPSPI_RX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_RX_INT_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_DisableTxInt
********************************************************************************
*
* Summary:
*  Disable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void DSPSPI_DisableTxInt(void) 
{
    #if(DSPSPI_INTERNAL_TX_INT_ENABLED)
        CyIntDisable(DSPSPI_TX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_TX_INT_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_DisableRxInt
********************************************************************************
*
* Summary:
*  Disable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void DSPSPI_DisableRxInt(void) 
{
    #if(DSPSPI_INTERNAL_RX_INT_ENABLED)
        CyIntDisable(DSPSPI_RX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_RX_INT_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_SetTxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void DSPSPI_SetTxInterruptMode(uint8 intSrc) 
{
    DSPSPI_TX_STATUS_MASK_REG  = intSrc;
}


/*******************************************************************************
* Function Name: DSPSPI_SetRxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void DSPSPI_SetRxInterruptMode(uint8 intSrc) 
{
    DSPSPI_RX_STATUS_MASK_REG  = intSrc;
}


/*******************************************************************************
* Function Name: DSPSPI_ReadTxStatus
********************************************************************************
*
* Summary:
*  Read the Tx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Tx status register.
*
* Global variables:
*  DSPSPI_swStatusTx - used to store in software status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Tx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 DSPSPI_ReadTxStatus(void) 
{
    uint8 tmpStatus;

    #if (DSPSPI_TX_SOFTWARE_BUF_ENABLED)

        DSPSPI_DisableTxInt();

        tmpStatus = DSPSPI_GET_STATUS_TX(DSPSPI_swStatusTx);

        DSPSPI_swStatusTx = 0u;

        /* Enable Interrupts */
        DSPSPI_EnableTxInt();

    #else /* !DSPSPI_TX_SOFTWARE_BUF_ENABLED */

        tmpStatus = DSPSPI_TX_STATUS_REG;

    #endif /* DSPSPI_TX_SOFTWARE_BUF_ENABLED */

    return(tmpStatus);
}


/*******************************************************************************
* Function Name: DSPSPI_ReadRxStatus
********************************************************************************
*
* Summary:
*  Read the Rx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Rx status register.
*
* Global variables:
*  DSPSPI_swStatusRx - used to store in software Rx status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Rx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Rx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 DSPSPI_ReadRxStatus(void) 
{
    uint8 tmpStatus;

    #if (DSPSPI_RX_SOFTWARE_BUF_ENABLED)

        DSPSPI_DisableRxInt();

        tmpStatus = DSPSPI_GET_STATUS_RX(DSPSPI_swStatusRx);

        DSPSPI_swStatusRx = 0u;

        /* Enable Interrupts */
        DSPSPI_EnableRxInt();

    #else /* !DSPSPI_RX_SOFTWARE_BUF_ENABLED */

        tmpStatus = DSPSPI_RX_STATUS_REG;

    #endif /* DSPSPI_RX_SOFTWARE_BUF_ENABLED */

    return(tmpStatus);
}


/*******************************************************************************
* Function Name: DSPSPI_WriteTxData
********************************************************************************
*
* Summary:
*  Write a byte of data to be sent across the SPI.
*
* Parameters:
*  txDataByte: The data value to send across the SPI.
*
* Return:
*  None.
*
* Global variables:
*  DSPSPI_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call if TX Software Buffer is used.
*  DSPSPI_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*  DSPSPI_txBuffer[DSPSPI_TX_BUFFER_SIZE] - used to store
*  data to sending, modified every function call if TX Software Buffer is used.
*
* Theory:
*  Allows the user to transmit any byte of data in a single transfer.
*
* Side Effects:
*  If this function is called again before the previous byte is finished then
*  the next byte will be appended to the transfer with no time between
*  the byte transfers. Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void DSPSPI_WriteTxData(uint8 txData) 
{
    #if(DSPSPI_TX_SOFTWARE_BUF_ENABLED)

        uint8 tempStatus;
        uint8 tmpTxBufferRead;

        /* Block if buffer is full, so we don't overwrite. */
        do
        {
            tmpTxBufferRead = DSPSPI_txBufferRead;
            if (0u == tmpTxBufferRead)
            {
                tmpTxBufferRead = DSPSPI_TX_BUFFER_SIZE - 1u;
            }
            else
            {
                tmpTxBufferRead--;
            }

        }
        while(tmpTxBufferRead == DSPSPI_txBufferWrite);

        /* Disable Interrupt to protect variables that could change on interrupt. */
        DSPSPI_DisableTxInt();

        tempStatus = DSPSPI_GET_STATUS_TX(DSPSPI_swStatusTx);
        DSPSPI_swStatusTx = tempStatus;

        if((DSPSPI_txBufferRead == DSPSPI_txBufferWrite) &&
            (0u != (DSPSPI_swStatusTx & DSPSPI_STS_TX_FIFO_NOT_FULL)))
        {
            /* Put data element into the TX FIFO */
            CY_SET_REG8(DSPSPI_TXDATA_PTR, txData);
        }
        else
        {
            /* Put data element into the software buffer */
            DSPSPI_txBufferWrite++;
            if(DSPSPI_txBufferWrite >= DSPSPI_TX_BUFFER_SIZE)
            {
                DSPSPI_txBufferWrite = 0u;
            }

            if(DSPSPI_txBufferWrite == DSPSPI_txBufferRead)
            {
                DSPSPI_txBufferRead++;
                if(DSPSPI_txBufferRead >= DSPSPI_RX_BUFFER_SIZE)
                {
                    DSPSPI_txBufferRead = 0u;
                }
                DSPSPI_txBufferFull = 1u;
            }

            DSPSPI_txBuffer[DSPSPI_txBufferWrite] = txData;

            DSPSPI_TX_STATUS_MASK_REG |= DSPSPI_STS_TX_FIFO_NOT_FULL;
        }

        /* Enable Interrupt. */
        DSPSPI_EnableTxInt();

    #else /* !DSPSPI_TX_SOFTWARE_BUF_ENABLED */
        /* Wait until TX FIFO has a place */
        while(0u == (DSPSPI_TX_STATUS_REG & DSPSPI_STS_TX_FIFO_NOT_FULL))
        {
        }

        /* Put data element into the TX FIFO */
        CY_SET_REG8(DSPSPI_TXDATA_PTR, txData);
        
    #endif /* DSPSPI_TX_SOFTWARE_BUF_ENABLED */
}

#if(DSPSPI_MODE_USE_ZERO != 0u)

    /*******************************************************************************
    * Function Name: DSPSPI_WriteTxDataZero
    ********************************************************************************
    *
    * Summary:
    *  Write a byte zero of data to be sent across the SPI. This must be used in
    *  Mode 00 and 01 if the FIFO is empty and data is not being sent.
    *
    * Parameters:
    *  txDataByte: The data value to send across the SPI.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer. Clear
    *  status register of the component.
    *
    *******************************************************************************/
    void DSPSPI_WriteTxDataZero(uint8 txDataByte)
                                        
    {
        CY_SET_REG8(DSPSPI_TXDATA_ZERO_PTR, txDataByte);
    }

#endif /* (DSPSPI_MODE_USE_ZERO != 0u) */


/*******************************************************************************
* Function Name: DSPSPI_ReadRxData
********************************************************************************
*
* Summary:
*  Read the next byte of data received across the SPI.
*
* Parameters:
*  None.
*
* Return:
*  The next byte of data read from the FIFO.
*
* Global variables:
*  DSPSPI_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  DSPSPI_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function
*  call if RX Software Buffer is used.
*  DSPSPI_rxBuffer[DSPSPI_RX_BUFFER_SIZE] - used to store
*  received data.
*
* Theory:
*  Allows the user to read a byte of data received.
*
* Side Effects:
*  Will return invalid data if the FIFO is empty.  User should poll for FIFO
*  empty status before calling Read function.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 DSPSPI_ReadRxData(void) 
{
    uint8 rxData;

    #if(DSPSPI_RX_SOFTWARE_BUF_ENABLED)

        /* Disable Interrupt to protect variables that could change on interrupt. */
        DSPSPI_DisableRxInt();

        if(DSPSPI_rxBufferRead != DSPSPI_rxBufferWrite)
        {
            if(DSPSPI_rxBufferFull == 0u)
            {
                DSPSPI_rxBufferRead++;
                if(DSPSPI_rxBufferRead >= DSPSPI_RX_BUFFER_SIZE)
                {
                    DSPSPI_rxBufferRead = 0u;
                }
            }
            else
            {
                DSPSPI_rxBufferFull = 0u;
            }
        }

        rxData = DSPSPI_rxBuffer[DSPSPI_rxBufferRead];

        /* Enable Interrupt. */
        DSPSPI_EnableRxInt();

    #else /* !DSPSPI_RX_SOFTWARE_BUF_ENABLED */

        rxData = CY_GET_REG8(DSPSPI_RXDATA_PTR);

    #endif /* DSPSPI_RX_SOFTWARE_BUF_ENABLED */

    return (rxData);

}


/*******************************************************************************
* Function Name: DSPSPI_GetRxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the RX buffer.
*  If RX Software Buffer not used then function return 0 if FIFO empty or 1 if
*  FIFO not empty. In another case function return size of RX Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the RX buffer.
*
* Global variables:
*  DSPSPI_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  DSPSPI_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer.
*
* Theory:
*  Allows the user to find out how full the RX Buffer is.
*
*******************************************************************************/
uint8 DSPSPI_GetRxBufferSize(void) 
{
    uint8 size;

    #if(DSPSPI_RX_SOFTWARE_BUF_ENABLED)

        /* Disable Interrupt to protect variables that could change on interrupt. */
        DSPSPI_DisableRxInt();

        if(DSPSPI_rxBufferRead == DSPSPI_rxBufferWrite)
        {
            size = 0u; /* No data in RX buffer */
        }
        else if(DSPSPI_rxBufferRead < DSPSPI_rxBufferWrite)
        {
            size = (DSPSPI_rxBufferWrite - DSPSPI_rxBufferRead);
        }
        else
        {
            size = (DSPSPI_RX_BUFFER_SIZE - DSPSPI_rxBufferRead) + DSPSPI_rxBufferWrite;
        }

        /* Enable interrupt. */
        DSPSPI_EnableRxInt();

    #else /* !DSPSPI_RX_SOFTWARE_BUF_ENABLED*/

        /* We can only know if there is data in the fifo. */
        size = ((DSPSPI_RX_STATUS_REG & DSPSPI_STS_RX_FIFO_NOT_EMPTY) != 0u) ? 1u : 0u;

    #endif /* DSPSPI_RX_SOFTWARE_BUF_ENABLED */

    return (size);
}


/*******************************************************************************
* Function Name: DSPSPI_GetTxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the TX buffer.
*  If TX Software Buffer not used then function return 0 - if FIFO empty, 1 - if
*  FIFO not full, 4 - if FIFO full. In another case function return size of TX
*  Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the TX buffer.
*
* Global variables:
*  DSPSPI_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  DSPSPI_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*
* Theory:
*  Allows the user to find out how full the TX Buffer is.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8  DSPSPI_GetTxBufferSize(void) 
{
    uint8 size;

    #if(DSPSPI_TX_SOFTWARE_BUF_ENABLED)

        /* Disable Interrupt to protect variables that could change on interrupt. */
        DSPSPI_DisableTxInt();

        if(DSPSPI_txBufferRead == DSPSPI_txBufferWrite)
        {
            size = 0u;
        }
        else if(DSPSPI_txBufferRead < DSPSPI_txBufferWrite)
        {
            size = (DSPSPI_txBufferWrite - DSPSPI_txBufferRead);
        }
        else
        {
            size = (DSPSPI_TX_BUFFER_SIZE - DSPSPI_txBufferRead) + DSPSPI_txBufferWrite;
        }

        /* Enable Interrupt. */
        DSPSPI_EnableTxInt();

    #else /* !DSPSPI_TX_SOFTWARE_BUF_ENABLED */

        size = DSPSPI_TX_STATUS_REG;

        /* Is the fifo is full. */
        if((size & DSPSPI_STS_TX_FIFO_EMPTY) != 0u)
        {
            size = 0u;
        }
        else if((size & DSPSPI_STS_TX_FIFO_NOT_FULL) != 0u)
        {
            size = 1u;
        }
        else
        {
            size = 4u;
        }

    #endif /* DSPSPI_TX_SOFTWARE_BUF_ENABLED */

    return (size);
}


/*******************************************************************************
* Function Name: DSPSPI_ClearRxBuffer
********************************************************************************
*
* Summary:
*  Clear the RX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  DSPSPI_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer, modified every function
*  call - resets to zero.
*  DSPSPI_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any received data not read from the RAM buffer will be lost when overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void DSPSPI_ClearRxBuffer(void) 
{
    /* Clear Hardware RX FIFO */
    while((DSPSPI_RX_STATUS_REG & DSPSPI_STS_RX_FIFO_NOT_EMPTY) != 0u)
    {
        CY_GET_REG8(DSPSPI_RXDATA_PTR);
    }

    #if(DSPSPI_RX_SOFTWARE_BUF_ENABLED)

        /* Disable interrupt to protect variables that could change on interrupt. */
        DSPSPI_DisableRxInt();

        DSPSPI_rxBufferRead  = 0u;
        DSPSPI_rxBufferWrite = 0u;
		DSPSPI_rxBufferFull  = 0u;
        /* Enable interrupt. */
        DSPSPI_EnableRxInt();

    #endif /* DSPSPI_RX_SOFTWARE_BUF_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_ClearTxBuffer
********************************************************************************
*
* Summary:
*  Clear the TX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  DSPSPI_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call - resets to zero.
*  DSPSPI_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any data not yet transmitted from the RAM buffer will be lost when
*  overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void DSPSPI_ClearTxBuffer(void) 
{
    uint8 enableInterrupts;

    /* Clear Hardware TX FIFO */
    enableInterrupts = CyEnterCriticalSection();

    #if(DSPSPI_DATA_WIDTH <= 8u)

        /* Clear TX FIFO */
        DSPSPI_TX_AUX_CONTROL_DP0_REG |= (uint8)DSPSPI_FIFO_CLR;
        DSPSPI_TX_AUX_CONTROL_DP0_REG &= (uint8)~DSPSPI_FIFO_CLR;

    #else

        /* Clear TX FIFO */
        DSPSPI_TX_AUX_CONTROL_DP0_REG |= (uint8)DSPSPI_FIFO_CLR;
        DSPSPI_TX_AUX_CONTROL_DP0_REG &= (uint8)~DSPSPI_FIFO_CLR;
        DSPSPI_TX_AUX_CONTROL_DP1_REG |= (uint8)DSPSPI_FIFO_CLR;
        DSPSPI_TX_AUX_CONTROL_DP1_REG &= (uint8)~DSPSPI_FIFO_CLR;

    #endif /* DSPSPI_DATA_WIDTH > 8u */

    CyExitCriticalSection(enableInterrupts);

    #if(DSPSPI_TX_SOFTWARE_BUF_ENABLED)

        /* Disable Interrupt to protect variables that could change on interrupt. */
        DSPSPI_DisableTxInt();

        DSPSPI_txBufferRead  = 0u;
        DSPSPI_txBufferWrite = 0u;
		DSPSPI_txBufferFull  = 0u;
		
        /* If Buffer is empty then disable TX FIFO status interrupt */
        DSPSPI_TX_STATUS_MASK_REG &= (uint8)~DSPSPI_STS_TX_FIFO_NOT_FULL;

        /* Enable Interrupt. */
        DSPSPI_EnableTxInt();

    #endif /* DSPSPI_TX_SOFTWARE_BUF_ENABLED */
}


#if (DSPSPI_BIDIRECTIONAL_MODE != 0u)

    /*******************************************************************************
    * Function Name: DSPSPI_TxEnable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI Slave is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to transmit.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void DSPSPI_TxEnable(void) 
    {
            DSPSPI_CONTROL_REG |= DSPSPI_CTRL_TX_SIGNAL_EN;
    }


    /*******************************************************************************
    * Function Name: DSPSPI_TxDisable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI Slave is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to receive.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void DSPSPI_TxDisable(void) 
    {
            DSPSPI_CONTROL_REG &= (uint8)~DSPSPI_CTRL_TX_SIGNAL_EN;
    }

#endif /* DSPSPI_BIDIRECTIONAL_MODE != 0u */


/*******************************************************************************
* Function Name: DSPSPI_PutArray
********************************************************************************
*
* Summary:
*  Write available data from RAM/ROM to the TX buffer while space is available
*  in the TX buffer. Keep trying until all data is passed to the TX buffer. If
*  used Mode 00 or 01 before PutArray() function should be called
*  WriteTxDataZero() function.
*
* Parameters:
*  *buffer: Pointer to the location in RAM containing the data to send
*  byteCount: The number of bytes to move to the transmit buffer.
*
* Return:
*  None.
*
* Side Effects:
*  Will stay in this routine until all data has been sent.  May get locked in
*  this loop if data is not being initiated by the master if there is not
*  enough room in the TX FIFO.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void DSPSPI_PutArray(const uint8 buffer[], uint8 byteCount)
                                                                    
{
    uint8 bufIndex;
    bufIndex = 0u;

    while(byteCount > 0u)
    {
        DSPSPI_WriteTxData(buffer[bufIndex]);
        bufIndex++;
        byteCount--;
    }
}


/*******************************************************************************
* Function Name: DSPSPI_ClearFIFO
********************************************************************************
*
* Summary:
*  Clear the RX and TX FIFO's of all data for a fresh start.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
void DSPSPI_ClearFIFO(void) 
{
    uint8 enableInterrupts;

    while((DSPSPI_RX_STATUS_REG & DSPSPI_STS_RX_FIFO_NOT_EMPTY) != 0u)
    {
        CY_GET_REG8(DSPSPI_RXDATA_PTR);
    }

    enableInterrupts = CyEnterCriticalSection();

    #if(DSPSPI_DATA_WIDTH <= 8u)

        /* Clear TX FIFO */
        DSPSPI_TX_AUX_CONTROL_DP0_REG |= (uint8)DSPSPI_FIFO_CLR;
        DSPSPI_TX_AUX_CONTROL_DP0_REG &= (uint8)~DSPSPI_FIFO_CLR;

    #else

        /* Clear TX FIFO */
        DSPSPI_TX_AUX_CONTROL_DP0_REG |= (uint8)DSPSPI_FIFO_CLR;
        DSPSPI_TX_AUX_CONTROL_DP0_REG &= (uint8)~DSPSPI_FIFO_CLR;
        DSPSPI_TX_AUX_CONTROL_DP1_REG |= (uint8)DSPSPI_FIFO_CLR;
        DSPSPI_TX_AUX_CONTROL_DP1_REG &= (uint8)~DSPSPI_FIFO_CLR;

    #endif /* DSPSPI_DATA_WIDTH <= 8u */

    CyExitCriticalSection(enableInterrupts);
}


/* Following functions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects.
*/


/*******************************************************************************
* Function Name: DSPSPI_EnableInt
********************************************************************************
*
* Summary:
*  Enable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void DSPSPI_EnableInt(void) 
{
    #if(DSPSPI_INTERNAL_TX_INT_ENABLED)
        CyIntEnable(DSPSPI_TX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_TX_INT_ENABLED */

    #if(DSPSPI_INTERNAL_RX_INT_ENABLED)
        CyIntEnable(DSPSPI_RX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_RX_INT_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_DisableInt
********************************************************************************
*
* Summary:
*  Disable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void DSPSPI_DisableInt(void) 
{
    #if(DSPSPI_INTERNAL_TX_INT_ENABLED)
        CyIntDisable(DSPSPI_TX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_TX_INT_ENABLED */

    #if(DSPSPI_INTERNAL_RX_INT_ENABLED)
        CyIntDisable(DSPSPI_RX_ISR_NUMBER);
    #endif /* DSPSPI_INTERNAL_RX_INT_ENABLED */
}


/*******************************************************************************
* Function Name: DSPSPI_SetInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void DSPSPI_SetInterruptMode(uint8 intSrc) 
{
    DSPSPI_TX_STATUS_MASK_REG  = intSrc;
    DSPSPI_RX_STATUS_MASK_REG  = intSrc;
}


/*******************************************************************************
* Function Name: DSPSPI_ReadStatus
********************************************************************************
*
* Summary:
*  Read the status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the status register.
*
* Global variables:
*  DSPSPI_swStatus - used to store in software status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the status register for error detection
*  and flow control.
*
* Side Effects:
*  Clear status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 DSPSPI_ReadStatus(void) 
{
    uint8 tmpStatus;

    #if ((DSPSPI_TX_SOFTWARE_BUF_ENABLED) || (DSPSPI_RX_SOFTWARE_BUF_ENABLED))

        DSPSPI_DisableInt();

        tmpStatus = DSPSPI_GET_STATUS_TX(DSPSPI_swStatusTx) |
              (DSPSPI_GET_STATUS_RX(DSPSPI_swStatusRx) & (uint8)~DSPSPI_STS_RX_FIFO_FULL);

        DSPSPI_swStatusTx = 0u;
        DSPSPI_swStatusRx = 0u;

        /* Enable Interrupts */
        DSPSPI_EnableInt();

    #else /* !((DSPSPI_TX_SOFTWARE_BUF_ENABLED) || (DSPSPI_RX_SOFTWARE_BUF_ENABLED)) */

        tmpStatus = DSPSPI_TX_STATUS_REG | (DSPSPI_RX_STATUS_REG &
                                                      (uint8)~DSPSPI_STS_RX_FIFO_FULL);

    #endif /* (DSPSPI_TX_SOFTWARE_BUF_ENABLED) || (DSPSPI_RX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/* [] END OF FILE */
