/*******************************************************************************
* File Name: DSPSPI.h
* Version 2.70
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Slave Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIS_DSPSPI_H)
#define CY_SPIS_DSPSPI_H

#include "cyfitter.h"
#include "cytypes.h"
#include "CyLib.h" /* For CyEnterCriticalSection() and CyExitCriticalSection() functions */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define DSPSPI_DATA_WIDTH                  (8u)
#define DSPSPI_INTERNAL_TX_INT_ENABLED     (0u)
#define DSPSPI_INTERNAL_RX_INT_ENABLED     (0u)
#define DSPSPI_MODE_USE_ZERO               (1u)
#define DSPSPI_BIDIRECTIONAL_MODE          (0u)
#define DSPSPI_MODE                        (0u)

#define DSPSPI_FIFO_SIZE                  (4u)
/* Internal interrupt handling */
#define DSPSPI_TX_BUFFER_SIZE             (4u)
#define DSPSPI_RX_BUFFER_SIZE             (4u)
#define DSPSPI_INTERNAL_TX_INT_ENABLED    (0u)
#define DSPSPI_INTERNAL_RX_INT_ENABLED    (0u)

#define DSPSPI_TX_SOFTWARE_BUF_ENABLED    ((0u != DSPSPI_INTERNAL_TX_INT_ENABLED) && \
                                                     (DSPSPI_TX_BUFFER_SIZE > DSPSPI_FIFO_SIZE))

#define DSPSPI_RX_SOFTWARE_BUF_ENABLED    ((0u != DSPSPI_INTERNAL_RX_INT_ENABLED) && \
                                                     (DSPSPI_RX_BUFFER_SIZE > DSPSPI_FIFO_SIZE))


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
    uint8 cntrPeriod;
} DSPSPI_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  DSPSPI_Init(void) ;
void  DSPSPI_Enable(void) ;
void  DSPSPI_Start(void) ;
void  DSPSPI_Stop(void) ;
void  DSPSPI_EnableTxInt(void) ;
void  DSPSPI_EnableRxInt(void) ;
void  DSPSPI_DisableTxInt(void) ;
void  DSPSPI_DisableRxInt(void) ;
void  DSPSPI_SetTxInterruptMode(uint8 intSrc) ;
void  DSPSPI_SetRxInterruptMode(uint8 intSrc) ;
uint8 DSPSPI_ReadTxStatus(void) ;
uint8 DSPSPI_ReadRxStatus(void) ;
void  DSPSPI_WriteTxData(uint8 txData);

#if(DSPSPI_MODE_USE_ZERO != 0u)
    void  DSPSPI_WriteTxDataZero(uint8 txDataByte) \
                                              ;
#endif /* (DSPSPI_MODE_USE_ZERO != 0u) */

uint8 DSPSPI_ReadRxData(void) ;
uint8 DSPSPI_GetRxBufferSize(void) ;
uint8 DSPSPI_GetTxBufferSize(void) ;
void  DSPSPI_ClearRxBuffer(void) ;
void  DSPSPI_ClearTxBuffer(void) ;

#if (DSPSPI_BIDIRECTIONAL_MODE != 0u)
    void  DSPSPI_TxEnable(void) ;
    void  DSPSPI_TxDisable(void) ;
#endif /* DSPSPI_BIDIRECTIONAL_MODE != 0u */

void  DSPSPI_PutArray(const uint8 buffer[], uint8 byteCount) ;
void  DSPSPI_ClearFIFO(void) ;
void  DSPSPI_Sleep(void) ;
void  DSPSPI_Wakeup(void) ;
void  DSPSPI_SaveConfig(void) ;
void  DSPSPI_RestoreConfig(void) ;

/* Communication bootloader APIs */
#if defined(CYDEV_BOOTLOADER_IO_COMP) && ((CYDEV_BOOTLOADER_IO_COMP == CyBtldr_DSPSPI) || \
                                          (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_Custom_Interface))
    /* Physical layer functions */
    void    DSPSPI_CyBtldrCommStart(void) CYSMALL ;
    void    DSPSPI_CyBtldrCommStop(void) CYSMALL ;
    void    DSPSPI_CyBtldrCommReset(void) CYSMALL ;
    cystatus DSPSPI_CyBtldrCommWrite(const uint8 pData[], uint16 size, uint16 * count, uint8 timeOut) CYSMALL
             ;
    cystatus DSPSPI_CyBtldrCommRead(uint8 pData[], uint16 size, uint16 * count, uint8 timeOut) CYSMALL
             ;

    #if (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_DSPSPI)
        #define CyBtldrCommStart    DSPSPI_CyBtldrCommStart
        #define CyBtldrCommStop     DSPSPI_CyBtldrCommStop
        #define CyBtldrCommReset    DSPSPI_CyBtldrCommReset
        #define CyBtldrCommWrite    DSPSPI_CyBtldrCommWrite
        #define CyBtldrCommRead     DSPSPI_CyBtldrCommRead
    #endif  /* (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_DSPSPI) */

    /* Byte to Byte time out for detecting end of block data from host */
    #define DSPSPI_BYTE2BYTE_TIME_OUT (1u)

#endif /* (CYDEV_BOOTLOADER_IO_COMP) && ((CYDEV_BOOTLOADER_IO_COMP == CyBtldr_DSPSPI) || \
                                             (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_Custom_Interface)) */


CY_ISR_PROTO(DSPSPI_TX_ISR);
CY_ISR_PROTO(DSPSPI_RX_ISR);

/* Macros for getting software status of SPIS Statusi Register */
#define DSPSPI_GET_STATUS_TX(swTxSts) ( (uint8)(DSPSPI_TX_STATUS_REG | \
                                                       ((swTxSts) & DSPSPI_STS_CLR_ON_RD_BYTES_MASK)) )
#define DSPSPI_GET_STATUS_RX(swRxSts) ( (uint8)(DSPSPI_RX_STATUS_REG | \
                                                       ((swRxSts) & DSPSPI_STS_CLR_ON_RD_BYTES_MASK)) )


/***************************************
*   Variable with external linkage
***************************************/

extern uint8 DSPSPI_initVar;


/***************************************
*           API Constants
***************************************/

#define DSPSPI_TX_ISR_NUMBER     ((uint8)DSPSPI_TxInternalInterrupt__INTC_NUMBER)
#define DSPSPI_RX_ISR_NUMBER     ((uint8)DSPSPI_RxInternalInterrupt__INTC_NUMBER)
#define DSPSPI_TX_ISR_PRIORITY   ((uint8)DSPSPI_TxInternalInterrupt__INTC_PRIOR_NUM)
#define DSPSPI_RX_ISR_PRIORITY   ((uint8)DSPSPI_RxInternalInterrupt__INTC_PRIOR_NUM)


/***************************************
*    Initial Parameter Constants
***************************************/

#define DSPSPI_INT_ON_SPI_DONE    (uint8)(0u << DSPSPI_STS_SPI_DONE_SHIFT)
#define DSPSPI_INT_ON_TX_EMPTY    (uint8)(0u << DSPSPI_STS_TX_FIFO_EMPTY_SHIFT)
#define DSPSPI_INT_ON_TX_NOT_FULL (uint8)(0u << DSPSPI_STS_TX_FIFO_NOT_FULL_SHIFT)
#define DSPSPI_INT_ON_BYTE_COMP   (uint8)(0u << DSPSPI_STS_BYTE_COMPLETE_SHIFT)

#define DSPSPI_TX_INIT_INTERRUPTS_MASK  (DSPSPI_INT_ON_SPI_DONE | \
                                            DSPSPI_INT_ON_TX_EMPTY | DSPSPI_INT_ON_TX_NOT_FULL | \
                                            DSPSPI_INT_ON_BYTE_COMP)

#define DSPSPI_INT_ON_RX_EMPTY     (uint8)(0u << DSPSPI_STS_RX_FIFO_EMPTY_SHIFT)
#define DSPSPI_INT_ON_RX_NOT_EMPTY (uint8)(0u << DSPSPI_STS_RX_FIFO_NOT_EMPTY_SHIFT)
#define DSPSPI_INT_ON_RX_OVER      (uint8)(0u << DSPSPI_STS_RX_FIFO_OVERRUN_SHIFT)
#define DSPSPI_INT_ON_RX_FULL      (uint8)(0u << DSPSPI_STS_RX_FIFO_FULL_SHIFT)

#define DSPSPI_RX_INIT_INTERRUPTS_MASK (DSPSPI_INT_ON_RX_EMPTY | \
                                            DSPSPI_INT_ON_RX_NOT_EMPTY | DSPSPI_INT_ON_RX_OVER | \
                                            DSPSPI_INT_ON_RX_FULL)

#define DSPSPI_BITCTR_INIT           (DSPSPI_DATA_WIDTH - 1u)

#define DSPSPI__MODE_00 0
#define DSPSPI__MODE_01 1
#define DSPSPI__MODE_10 2
#define DSPSPI__MODE_11 3


#define DSPSPI_TX_BUFFER_SIZE         (4u)
#define DSPSPI_RX_BUFFER_SIZE         (4u)

/* Following definitions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects
*/
#define DSPSPI_INIT_INTERRUPTS_MASK  (DSPSPI_INT_ON_SPI_DONE | DSPSPI_INT_ON_TX_EMPTY | \
                                            DSPSPI_INT_ON_TX_NOT_FULL | DSPSPI_INT_ON_RX_EMPTY | \
                                            DSPSPI_INT_ON_RX_NOT_EMPTY | DSPSPI_INT_ON_RX_OVER | \
                                            DSPSPI_INT_ON_BYTE_COMP)


/***************************************
*             Registers
***************************************/
#if(CY_PSOC3 || CY_PSOC5)
    #define DSPSPI_TXDATA_ZERO_REG          (* (reg8  *) \
            DSPSPI_BSPIS_sR8_Dp_u0__A0_REG)

    #define DSPSPI_TXDATA_ZERO_PTR           (  (reg8  *) \
            DSPSPI_BSPIS_sR8_Dp_u0__A0_REG)

    #define DSPSPI_RXDATA_ZERO_REG           (* (reg8  *) \
            DSPSPI_BSPIS_sR8_Dp_u0__A0_REG)

    #define DSPSPI_RXDATA_ZERO_PTR           (  (reg8  *) \
            DSPSPI_BSPIS_sR8_Dp_u0__A0_REG)

    #define DSPSPI_TXDATA_REG                (* (reg8  *) \
            DSPSPI_BSPIS_sR8_Dp_u0__F0_REG)

    #define DSPSPI_TXDATA_PTR                (  (reg8  *) \
            DSPSPI_BSPIS_sR8_Dp_u0__F0_REG)

    #define DSPSPI_RXDATA_REG                (* (reg8  *) \
            DSPSPI_BSPIS_sR8_Dp_u0__F1_REG)

    #define DSPSPI_RXDATA_PTR                (  (reg8  *) \
            DSPSPI_BSPIS_sR8_Dp_u0__F1_REG)
#else
    #if(DSPSPI_DATA_WIDTH <= 8u) /* 8bit - SPIS */
        #define DSPSPI_TXDATA_ZERO_REG           (* (reg8 *) \
                DSPSPI_BSPIS_sR8_Dp_u0__A0_REG)

        #define DSPSPI_TXDATA_ZERO_PTR           (  (reg8  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__A0_REG)

        #define DSPSPI_RXDATA_ZERO_REG           (* (reg8  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__A0_REG)

        #define DSPSPI_RXDATA_ZERO_PTR           (  (reg8 *) \
                DSPSPI_BSPIS_sR8_Dp_u0__A0_REG)

        #define DSPSPI_TXDATA_REG                (* (reg8  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__F0_REG)

        #define DSPSPI_TXDATA_PTR                (  (reg8  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__F0_REG)

        #define DSPSPI_RXDATA_REG                (* (reg8  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__F1_REG)

        #define DSPSPI_RXDATA_PTR                (  (reg8  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__F1_REG)
    #else /* 16bit - SPIS */
        #define DSPSPI_TXDATA_ZERO_REG           (* (reg16  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__16BIT_A0_REG)

        #define DSPSPI_TXDATA_ZERO_PTR           (  (reg16  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__16BIT_A0_REG)

        #define DSPSPI_RXDATA_ZERO_REG           (* (reg16  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__16BIT_A0_REG)

        #define DSPSPI_RXDATA_ZERO_PTR           (  (reg16  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__16BIT_A0_REG)

        #define DSPSPI_TXDATA_REG                (* (reg16  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__16BIT_F0_REG)

        #define DSPSPI_TXDATA_PTR                (  (reg16  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__16BIT_F0_REG)

        #define DSPSPI_RXDATA_REG                (* (reg16  *) \
                DSPSPI_BSPIS_sR8_Dp_u0__16BIT_F1_REG)

        #define DSPSPI_RXDATA_PTR                (  (reg16 *) \
                DSPSPI_BSPIS_sR8_Dp_u0__16BIT_F1_REG)
    #endif /* (DSPSPI_DATA_WIDTH <= 8u) */
#endif     /* (CY_PSOC3 || CY_PSOC5) */

#define DSPSPI_TX_AUX_CONTROL_DP0_REG       (* (reg8 *) \
        DSPSPI_BSPIS_sR8_Dp_u0__DP_AUX_CTL_REG)
#define DSPSPI_TX_AUX_CONTROL_DP0_PTR       (  (reg8 *) \
        DSPSPI_BSPIS_sR8_Dp_u0__DP_AUX_CTL_REG)

#define DSPSPI_RX_AUX_CONTROL_DP0_REG       (* (reg8 *) \
        DSPSPI_BSPIS_sR8_Dp_u0__DP_AUX_CTL_REG)
#define DSPSPI_RX_AUX_CONTROL_DP0_PTR       (  (reg8 *) \
        DSPSPI_BSPIS_sR8_Dp_u0__DP_AUX_CTL_REG)

#if(DSPSPI_DATA_WIDTH > 8u)

    #define DSPSPI_TX_AUX_CONTROL_DP1_REG   (* (reg8 *) \
            DSPSPI_BSPIS_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define DSPSPI_TX_AUX_CONTROL_DP1_PTR   (  (reg8 *) \
            DSPSPI_BSPIS_sR8_Dp_u1__DP_AUX_CTL_REG)

    #define DSPSPI_RX_AUX_CONTROL_DP1_REG   (* (reg8 *) \
            DSPSPI_BSPIS_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define DSPSPI_RX_AUX_CONTROL_DP1_PTR   (  (reg8 *) \
            DSPSPI_BSPIS_sR8_Dp_u1__DP_AUX_CTL_REG)

#endif /* DSPSPI_DATA_WIDTH > 8u */


#define DSPSPI_COUNTER_PERIOD_REG    (* (reg8 *) \
        DSPSPI_BSPIS_BitCounter__PERIOD_REG)
#define DSPSPI_COUNTER_PERIOD_PTR    (  (reg8 *) \
        DSPSPI_BSPIS_BitCounter__PERIOD_REG)

#define DSPSPI_TX_STATUS_MASK_REG    (* (reg8 *) \
        DSPSPI_BSPIS_TxStsReg__MASK_REG)
#define DSPSPI_TX_STATUS_MASK_PTR    (  (reg8 *) \
        DSPSPI_BSPIS_TxStsReg__MASK_REG)

#define DSPSPI_RX_STATUS_MASK_REG    (* (reg8 *) \
        DSPSPI_BSPIS_RxStsReg__MASK_REG)
#define DSPSPI_RX_STATUS_MASK_PTR    (  (reg8 *) \
        DSPSPI_BSPIS_RxStsReg__MASK_REG)

#define DSPSPI_ONE_REG               (* (reg8 *) \
        DSPSPI_BSPIS_SPISlave_dpCounter_u0__D1_REG)
#define DSPSPI_ONE_PTR               (  (reg8 *) \
        DSPSPI_BSPIS_dpCounter_u0__D1_REG)

#define DSPSPI_TX_STATUS_REG         (* (reg8 *) \
        DSPSPI_BSPIS_TxStsReg__STATUS_REG)
#define DSPSPI_TX_STATUS_PTR         (  (reg8 *) \
        DSPSPI_BSPIS__TxStsReg__STATUS_REG)

#define DSPSPI_RX_STATUS_REG         (* (reg8 *) \
        DSPSPI_BSPIS_RxStsReg__STATUS_REG)
#define DSPSPI_RX_STATUS_PTR         (  (reg8 *) \
        DSPSPI_BSPIS_RxStsReg__STATUS_REG)

#define DSPSPI_COUNTER_CONTROL_REG   (* (reg8 *) \
        DSPSPI_BSPIS_BitCounter__CONTROL_AUX_CTL_REG)
#define DSPSPI_COUNTER_CONTROL_PTR   (  (reg8 *) \
        DSPSPI_BSPIS_BitCounter__CONTROL_AUX_CTL_REG)

#define DSPSPI_TX_STATUS_ACTL_REG    (* (reg8 *) \
        DSPSPI_BSPIS_TxStsReg__STATUS_AUX_CTL_REG)
#define DSPSPI_TX_STATUS_ACTL_PTR    (  (reg8 *) \
        DSPSPI_TX_BSPIS_TxStsReg__STATUS_AUX_CTL_REG)

#define DSPSPI_RX_STATUS_ACTL_REG    (* (reg8 *) \
        DSPSPI_BSPIS_RxStsReg__STATUS_AUX_CTL_REG)
#define DSPSPI_RX_STATUS_ACTL_PTR    (  (reg8 *) \
        DSPSPI_RX_BSPIS_RxStsReg__STATUS_AUX_CTL_REG)

#if(DSPSPI_BIDIRECTIONAL_MODE)

    #define DSPSPI_CONTROL_REG       (* (reg8 *) \
   DSPSPI_BSPIS_SyncCtl_CtrlReg__CONTROL_REG)
    #define DSPSPI_CONTROL_PTR       (  (reg8 *) \
   DSPSPI_BSPIS_SyncCtl_CtrlReg__CONTROL_REG)

#endif /* DSPSPI_BIDIRECTIONAL_MODE */


/***************************************
*       Register Constants
***************************************/

/* Status Register Definitions */
#define DSPSPI_STS_SPI_DONE_SHIFT             (0x00u)
#define DSPSPI_STS_TX_FIFO_NOT_FULL_SHIFT     (0x01u)
#define DSPSPI_STS_TX_FIFO_EMPTY_SHIFT        (0x02u)
#define DSPSPI_STS_RX_FIFO_NOT_EMPTY_SHIFT    (0x03u)
#define DSPSPI_STS_RX_FIFO_EMPTY_SHIFT        (0x04u)
#define DSPSPI_STS_RX_FIFO_OVERRUN_SHIFT      (0x05u)
#define DSPSPI_STS_RX_FIFO_FULL_SHIFT         (0x06u)
#define DSPSPI_STS_BYTE_COMPLETE_SHIFT        (0x06u)

#define DSPSPI_STS_SPI_DONE                   ((uint8)(0x01u << DSPSPI_STS_SPI_DONE_SHIFT))
#define DSPSPI_STS_TX_FIFO_EMPTY              ((uint8)(0x01u << DSPSPI_STS_TX_FIFO_EMPTY_SHIFT))
#define DSPSPI_STS_TX_FIFO_NOT_FULL           ((uint8)(0x01u << DSPSPI_STS_TX_FIFO_NOT_FULL_SHIFT))
#define DSPSPI_STS_RX_FIFO_EMPTY              ((uint8)(0x01u << DSPSPI_STS_RX_FIFO_EMPTY_SHIFT))
#define DSPSPI_STS_RX_FIFO_NOT_EMPTY          ((uint8)(0x01u << DSPSPI_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define DSPSPI_STS_RX_FIFO_OVERRUN            ((uint8)(0x01u << DSPSPI_STS_RX_FIFO_OVERRUN_SHIFT))
#define DSPSPI_STS_RX_FIFO_FULL               ((uint8)(0x01u << DSPSPI_STS_RX_FIFO_FULL_SHIFT))
#define DSPSPI_STS_BYTE_COMPLETE              ((uint8)(0x01u << DSPSPI_STS_BYTE_COMPLETE_SHIFT))

#define DSPSPI_STS_CLR_ON_RD_BYTES_MASK       (0x61u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define DSPSPI_INT_ENABLE                     (0x10u)
#define DSPSPI_TX_FIFO_CLR    (0x01u) /* F0 - TX FIFO */
#define DSPSPI_RX_FIFO_CLR    (0x02u) /* F1 - RX FIFO */
#define DSPSPI_FIFO_CLR       (DSPSPI_TX_FIFO_CLR | DSPSPI_RX_FIFO_CLR)

/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define DSPSPI_CNTR_ENABLE                    (0x20u)

/* Bi-Directional mode control bit */
#define DSPSPI_CTRL_TX_SIGNAL_EN              (0x01u)

/* Datapath Auxillary Control Register definitions */
#define DSPSPI_AUX_CTRL_FIFO0_CLR             (0x00u)
#define DSPSPI_AUX_CTRL_FIFO1_CLR             (0x00u)
#define DSPSPI_AUX_CTRL_FIFO0_LVL             (0x00u)
#define DSPSPI_AUX_CTRL_FIFO1_LVL             (0x00u)
#define DSPSPI_STATUS_ACTL_INT_EN_MASK        (0x10u)

/* Component disabled */
#define DSPSPI_DISABLED   (0u)

/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define DSPSPI_TXDATA_ZERO               (DSPSPI_TXDATA_ZERO_REG)
#define DSPSPI_TXDATA                    (DSPSPI_TXDATA_REG)
#define DSPSPI_RXDATA                    (DSPSPI_RXDATA_REG)
#define DSPSPI_MISO_AUX_CONTROLDP0       (DSPSPI_MISO_AUX_CTRL_DP0_REG)
#define DSPSPI_MOSI_AUX_CONTROLDP0       (DSPSPI_MOSI_AUX_CTRL_DP0_REG)
#define DSPSPI_TXBUFFERREAD              (DSPSPI_txBufferRead)
#define DSPSPI_TXBUFFERWRITE             (DSPSPI_txBufferWrite)
#define DSPSPI_RXBUFFERREAD              (DSPSPI_rxBufferRead)
#define DSPSPI_RXBUFFERWRITE             (DSPSPI_rxBufferWrite)

#if(DSPSPI_DATA_WIDTH > 8u)

    #define DSPSPI_MISO_AUX_CONTROLDP1   (DSPSPI_MISO_AUX_CTRL_DP1_REG)
    #define DSPSPI_MOSI_AUX_CONTROLDP1   (DSPSPI_MOSI_AUX_CTRL_DP0_REG)

#endif /* DSPSPI_DATA_WIDTH > 8u */

#define DSPSPI_COUNTER_PERIOD            (DSPSPI_COUNTER_PERIOD_REG)
#define DSPSPI_COUNTER_CONTROL           (DSPSPI_COUNTER_CONTROL_REG)
#define DSPSPI_ONE                       (DSPSPI_ONE_REG)
#define DSPSPI_STATUS                    (DSPSPI_TX_STATUS_REG)
#define DSPSPI_STATUS_MASK               (DSPSPI_TX_STATUS_MASK_REG)
#define DSPSPI_STATUS_ACTL               (DSPSPI_TX_STATUS_ACTL_REG)

#define DSPSPI_WriteByte      (DSPSPI_WriteTxData)
#define DSPSPI_ReadByte       (DSPSPI_ReadRxData)
#define DSPSPI_WriteByteZero  (DSPSPI_WriteTxDataZero)
void  DSPSPI_SetInterruptMode(uint8 intSrc) ;
uint8 DSPSPI_ReadStatus(void) ;
void  DSPSPI_EnableInt(void) ;
void  DSPSPI_DisableInt(void) ;

#define DSPSPI_STS_TX_BUF_NOT_FULL_SHIFT      (0x01u)
#define DSPSPI_STS_TX_BUF_FULL_SHIFT          (0x02u)
#define DSPSPI_STS_RX_BUF_NOT_EMPTY_SHIFT     (0x03u)
#define DSPSPI_STS_RX_BUF_EMPTY_SHIFT         (0x04u)
#define DSPSPI_STS_RX_BUF_OVERRUN_SHIFT       (0x05u)

#define DSPSPI_STS_TX_BUF_NOT_FULL            ((uint8)(0x01u << DSPSPI_STS_TX_BUF_NOT_FULL_SHIFT))
#define DSPSPI_STS_TX_BUF_FULL                ((uint8)(0x01u << DSPSPI_STS_TX_BUF_FULL_SHIFT))
#define DSPSPI_STS_RX_BUF_NOT_EMPTY           ((uint8)(0x01u << DSPSPI_STS_RX_BUF_NOT_EMPTY_SHIFT))
#define DSPSPI_STS_RX_BUF_EMPTY               ((uint8)(0x01u << DSPSPI_STS_RX_BUF_EMPTY_SHIFT))
#define DSPSPI_STS_RX_BUF_OVERRUN             ((uint8)(0x01u << DSPSPI_STS_RX_BUF_OVERRUN_SHIFT))

#define DSPSPI_DataWidth                  (DSPSPI_DATA_WIDTH)
#define DSPSPI_InternalClockUsed          (DSPSPI_INTERNAL_CLOCK)
#define DSPSPI_InternalTxInterruptEnabled (DSPSPI_INTERNAL_TX_INT_ENABLED)
#define DSPSPI_InternalRxInterruptEnabled (DSPSPI_INTERNAL_RX_INT_ENABLED)
#define DSPSPI_ModeUseZero                (DSPSPI_MODE_USE_ZERO)
#define DSPSPI_BidirectionalMode          (DSPSPI_BIDIRECTIONAL_MODE)
#define DSPSPI_Mode                       (DSPSPI_MODE)
#define DSPSPI_DATAWIDHT                  (DSPSPI_DATA_WIDTH)
#define DSPSPI_InternalInterruptEnabled   (0u)

#define DSPSPI_TXBUFFERSIZE   (DSPSPI_TX_BUFFER_SIZE)
#define DSPSPI_RXBUFFERSIZE   (DSPSPI_RX_BUFFER_SIZE)

#define DSPSPI_TXBUFFER       DSPSPI_txBuffer
#define DSPSPI_RXBUFFER       DSPSPI_rxBuffer

#endif  /* CY_SPIS_DSPSPI_H */

/* [] END OF FILE */
