/*******************************************************************************
* File Name: DSPSPI_INT.c
* Version 2.70
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Slave
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "DSPSPI_PVT.h"
#include "cyapicallbacks.h"

/* User code required at start of ISR */
/* `#START DSPSPI_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: DSPSPI_TX_ISR
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Slave.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  DSPSPI_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  DSPSPI_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  DSPSPI_txBuffer[DSPSPI_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(DSPSPI_TX_ISR)
{

    #if(DSPSPI_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (DSPSPI_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef DSPSPI_TX_ISR_ENTRY_CALLBACK
        DSPSPI_TX_ISR_EntryCallback();
    #endif /* DSPSPI_TX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START DSPSPI_ISR_TX_START` */

    /* `#END` */

    #if(DSPSPI_TX_SOFTWARE_BUF_ENABLED)
        /* Component interrupt service code */

        /* See if TX data buffer is not empty and there is space in TX FIFO */
        while(DSPSPI_txBufferRead != DSPSPI_txBufferWrite)
        {
            tmpStatus = DSPSPI_GET_STATUS_TX(DSPSPI_swStatusTx);
            DSPSPI_swStatusTx = tmpStatus;

            if ((DSPSPI_swStatusTx & DSPSPI_STS_TX_FIFO_NOT_FULL) != 0u)
            {
                if(DSPSPI_txBufferFull == 0u)
                {
                   DSPSPI_txBufferRead++;

                    if(DSPSPI_txBufferRead >= DSPSPI_TX_BUFFER_SIZE)
                    {
                        DSPSPI_txBufferRead = 0u;
                    }
                }
                else
                {
                    DSPSPI_txBufferFull = 0u;
                }

                /* Put data element into the TX FIFO */
                CY_SET_REG8(DSPSPI_TXDATA_PTR, 
                                             DSPSPI_txBuffer[DSPSPI_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        /* If Buffer is empty then disable TX FIFO status interrupt until there is data in the buffer */
        if(DSPSPI_txBufferRead == DSPSPI_txBufferWrite)
        {
            DSPSPI_TX_STATUS_MASK_REG &= ((uint8)~DSPSPI_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* DSPSPI_TX_SOFTWARE_BUF_ENABLED */

    /* User code required at end of ISR (Optional) */
    /* `#START DSPSPI_ISR_TX_END` */

    /* `#END` */
    
    #ifdef DSPSPI_TX_ISR_EXIT_CALLBACK
        DSPSPI_TX_ISR_ExitCallback();
    #endif /* DSPSPI_TX_ISR_EXIT_CALLBACK */
   }


/*******************************************************************************
* Function Name: DSPSPI_RX_ISR
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Slave.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  DSPSPI_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  DSPSPI_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  DSPSPI_rxBuffer[DSPSPI_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(DSPSPI_RX_ISR)
{
    #if(DSPSPI_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (DSPSPI_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef DSPSPI_RX_ISR_ENTRY_CALLBACK
        DSPSPI_RX_ISR_EntryCallback();
    #endif /* DSPSPI_RX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START DSPSPI_RX_ISR_START` */

    /* `#END` */
    
    #if(DSPSPI_RX_SOFTWARE_BUF_ENABLED)
        tmpStatus = DSPSPI_GET_STATUS_RX(DSPSPI_swStatusRx);
        DSPSPI_swStatusRx = tmpStatus;
        /* See if RX data FIFO has some data and if it can be moved to the RX Buffer */
        while((DSPSPI_swStatusRx & DSPSPI_STS_RX_FIFO_NOT_EMPTY) != 0u)
        {
            rxData = CY_GET_REG8(DSPSPI_RXDATA_PTR);

            /* Set next pointer. */
            DSPSPI_rxBufferWrite++;
            if(DSPSPI_rxBufferWrite >= DSPSPI_RX_BUFFER_SIZE)
            {
                DSPSPI_rxBufferWrite = 0u;
            }

            if(DSPSPI_rxBufferWrite == DSPSPI_rxBufferRead)
            {
                DSPSPI_rxBufferRead++;
                if(DSPSPI_rxBufferRead >= DSPSPI_RX_BUFFER_SIZE)
                {
                    DSPSPI_rxBufferRead = 0u;
                }
                DSPSPI_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            DSPSPI_rxBuffer[DSPSPI_rxBufferWrite] = rxData;

            tmpStatus = DSPSPI_GET_STATUS_RX(DSPSPI_swStatusRx);
            DSPSPI_swStatusRx = tmpStatus;
        }
    #endif /* DSPSPI_RX_SOFTWARE_BUF_ENABLED */

    /* User code required at end of ISR (Optional) */
    /* `#START DSPSPI_RX_ISR_END` */

    /* `#END` */

    #ifdef DSPSPI_RX_ISR_EXIT_CALLBACK
        DSPSPI_RX_ISR_ExitCallback();
    #endif /* DSPSPI_RX_ISR_EXIT_CALLBACK */
}

/* [] END OF FILE */
