/*******************************************************************************
* File Name: LEDDRIVER.c
* Version 2.50
*
* Description:
*  This file provides all API functionality of the SPI Master component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "LEDDRIVER_PVT.h"

#if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED)
    volatile uint16 LEDDRIVER_txBuffer[LEDDRIVER_TX_BUFFER_SIZE];
    volatile uint8 LEDDRIVER_txBufferFull;
    volatile uint8 LEDDRIVER_txBufferRead;
    volatile uint8 LEDDRIVER_txBufferWrite;
#endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */

#if(LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)
    volatile uint16 LEDDRIVER_rxBuffer[LEDDRIVER_RX_BUFFER_SIZE];
    volatile uint8 LEDDRIVER_rxBufferFull;
    volatile uint8 LEDDRIVER_rxBufferRead;
    volatile uint8 LEDDRIVER_rxBufferWrite;
#endif /* (LEDDRIVER_RX_SOFTWARE_BUF_ENABLED) */

uint8 LEDDRIVER_initVar = 0u;

volatile uint8 LEDDRIVER_swStatusTx;
volatile uint8 LEDDRIVER_swStatusRx;


/*******************************************************************************
* Function Name: LEDDRIVER_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default SPIM configuration provided with customizer.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  When this function is called it initializes all of the necessary parameters
*  for execution. i.e. setting the initial interrupt mask, configuring the
*  interrupt service routine, configuring the bit-counter parameters and
*  clearing the FIFO and Status Register.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void LEDDRIVER_Init(void) 
{
    /* Initialize the Bit counter */
    LEDDRIVER_COUNTER_PERIOD_REG = LEDDRIVER_BITCTR_INIT;

    /* Init TX ISR  */
    #if(0u != LEDDRIVER_INTERNAL_TX_INT_ENABLED)
        CyIntDisable         (LEDDRIVER_TX_ISR_NUMBER);
        CyIntSetPriority     (LEDDRIVER_TX_ISR_NUMBER,  LEDDRIVER_TX_ISR_PRIORITY);
        (void) CyIntSetVector(LEDDRIVER_TX_ISR_NUMBER, &LEDDRIVER_TX_ISR);
    #endif /* (0u != LEDDRIVER_INTERNAL_TX_INT_ENABLED) */

    /* Init RX ISR  */
    #if(0u != LEDDRIVER_INTERNAL_RX_INT_ENABLED)
        CyIntDisable         (LEDDRIVER_RX_ISR_NUMBER);
        CyIntSetPriority     (LEDDRIVER_RX_ISR_NUMBER,  LEDDRIVER_RX_ISR_PRIORITY);
        (void) CyIntSetVector(LEDDRIVER_RX_ISR_NUMBER, &LEDDRIVER_RX_ISR);
    #endif /* (0u != LEDDRIVER_INTERNAL_RX_INT_ENABLED) */

    /* Clear any stray data from the RX and TX FIFO */
    LEDDRIVER_ClearFIFO();

    #if(LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)
        LEDDRIVER_rxBufferFull  = 0u;
        LEDDRIVER_rxBufferRead  = 0u;
        LEDDRIVER_rxBufferWrite = 0u;
    #endif /* (LEDDRIVER_RX_SOFTWARE_BUF_ENABLED) */

    #if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED)
        LEDDRIVER_txBufferFull  = 0u;
        LEDDRIVER_txBufferRead  = 0u;
        LEDDRIVER_txBufferWrite = 0u;
    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */

    (void) LEDDRIVER_ReadTxStatus(); /* Clear Tx status and swStatusTx */
    (void) LEDDRIVER_ReadRxStatus(); /* Clear Rx status and swStatusRx */

    /* Configure TX and RX interrupt mask */
    LEDDRIVER_TX_STATUS_MASK_REG = LEDDRIVER_TX_INIT_INTERRUPTS_MASK;
    LEDDRIVER_RX_STATUS_MASK_REG = LEDDRIVER_RX_INIT_INTERRUPTS_MASK;
}


/*******************************************************************************
* Function Name: LEDDRIVER_Enable
********************************************************************************
*
* Summary:
*  Enable SPIM component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void LEDDRIVER_Enable(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    LEDDRIVER_COUNTER_CONTROL_REG |= LEDDRIVER_CNTR_ENABLE;
    LEDDRIVER_TX_STATUS_ACTL_REG  |= LEDDRIVER_INT_ENABLE;
    LEDDRIVER_RX_STATUS_ACTL_REG  |= LEDDRIVER_INT_ENABLE;
    CyExitCriticalSection(enableInterrupts);

    #if(0u != LEDDRIVER_INTERNAL_CLOCK)
        LEDDRIVER_IntClock_Enable();
    #endif /* (0u != LEDDRIVER_INTERNAL_CLOCK) */

    LEDDRIVER_EnableTxInt();
    LEDDRIVER_EnableRxInt();
}


/*******************************************************************************
* Function Name: LEDDRIVER_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the SPI Master component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  LEDDRIVER_initVar - used to check initial configuration, modified on
*  first function call.
*
* Theory:
*  Enable the clock input to enable operation.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void LEDDRIVER_Start(void) 
{
    if(0u == LEDDRIVER_initVar)
    {
        LEDDRIVER_Init();
        LEDDRIVER_initVar = 1u;
    }

    LEDDRIVER_Enable();
}


/*******************************************************************************
* Function Name: LEDDRIVER_Stop
********************************************************************************
*
* Summary:
*  Disable the SPI Master component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the clock input to enable operation.
*
*******************************************************************************/
void LEDDRIVER_Stop(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    LEDDRIVER_TX_STATUS_ACTL_REG &= ((uint8) ~LEDDRIVER_INT_ENABLE);
    LEDDRIVER_RX_STATUS_ACTL_REG &= ((uint8) ~LEDDRIVER_INT_ENABLE);
    CyExitCriticalSection(enableInterrupts);

    #if(0u != LEDDRIVER_INTERNAL_CLOCK)
        LEDDRIVER_IntClock_Disable();
    #endif /* (0u != LEDDRIVER_INTERNAL_CLOCK) */

    LEDDRIVER_DisableTxInt();
    LEDDRIVER_DisableRxInt();
}


/*******************************************************************************
* Function Name: LEDDRIVER_EnableTxInt
********************************************************************************
*
* Summary:
*  Enable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void LEDDRIVER_EnableTxInt(void) 
{
    #if(0u != LEDDRIVER_INTERNAL_TX_INT_ENABLED)
        CyIntEnable(LEDDRIVER_TX_ISR_NUMBER);
    #endif /* (0u != LEDDRIVER_INTERNAL_TX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: LEDDRIVER_EnableRxInt
********************************************************************************
*
* Summary:
*  Enable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void LEDDRIVER_EnableRxInt(void) 
{
    #if(0u != LEDDRIVER_INTERNAL_RX_INT_ENABLED)
        CyIntEnable(LEDDRIVER_RX_ISR_NUMBER);
    #endif /* (0u != LEDDRIVER_INTERNAL_RX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: LEDDRIVER_DisableTxInt
********************************************************************************
*
* Summary:
*  Disable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void LEDDRIVER_DisableTxInt(void) 
{
    #if(0u != LEDDRIVER_INTERNAL_TX_INT_ENABLED)
        CyIntDisable(LEDDRIVER_TX_ISR_NUMBER);
    #endif /* (0u != LEDDRIVER_INTERNAL_TX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: LEDDRIVER_DisableRxInt
********************************************************************************
*
* Summary:
*  Disable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void LEDDRIVER_DisableRxInt(void) 
{
    #if(0u != LEDDRIVER_INTERNAL_RX_INT_ENABLED)
        CyIntDisable(LEDDRIVER_RX_ISR_NUMBER);
    #endif /* (0u != LEDDRIVER_INTERNAL_RX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: LEDDRIVER_SetTxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void LEDDRIVER_SetTxInterruptMode(uint8 intSrc) 
{
    LEDDRIVER_TX_STATUS_MASK_REG = intSrc;
}


/*******************************************************************************
* Function Name: LEDDRIVER_SetRxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void LEDDRIVER_SetRxInterruptMode(uint8 intSrc) 
{
    LEDDRIVER_RX_STATUS_MASK_REG  = intSrc;
}


/*******************************************************************************
* Function Name: LEDDRIVER_ReadTxStatus
********************************************************************************
*
* Summary:
*  Read the Tx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Tx status register.
*
* Global variables:
*  LEDDRIVER_swStatusTx - used to store in software status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Tx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 LEDDRIVER_ReadTxStatus(void) 
{
    uint8 tmpStatus;

    #if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        LEDDRIVER_DisableTxInt();

        tmpStatus = LEDDRIVER_GET_STATUS_TX(LEDDRIVER_swStatusTx);
        LEDDRIVER_swStatusTx = 0u;

        LEDDRIVER_EnableTxInt();

    #else

        tmpStatus = LEDDRIVER_TX_STATUS_REG;

    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/*******************************************************************************
* Function Name: LEDDRIVER_ReadRxStatus
********************************************************************************
*
* Summary:
*  Read the Rx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Rx status register.
*
* Global variables:
*  LEDDRIVER_swStatusRx - used to store in software Rx status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Rx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Rx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 LEDDRIVER_ReadRxStatus(void) 
{
    uint8 tmpStatus;

    #if(LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)
        /* Disable RX interrupt to protect global veriables */
        LEDDRIVER_DisableRxInt();

        tmpStatus = LEDDRIVER_GET_STATUS_RX(LEDDRIVER_swStatusRx);
        LEDDRIVER_swStatusRx = 0u;

        LEDDRIVER_EnableRxInt();

    #else

        tmpStatus = LEDDRIVER_RX_STATUS_REG;

    #endif /* (LEDDRIVER_RX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/*******************************************************************************
* Function Name: LEDDRIVER_WriteTxData
********************************************************************************
*
* Summary:
*  Write a byte of data to be sent across the SPI.
*
* Parameters:
*  txDataByte: The data value to send across the SPI.
*
* Return:
*  None.
*
* Global variables:
*  LEDDRIVER_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call if TX Software Buffer is used.
*  LEDDRIVER_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*  LEDDRIVER_txBuffer[LEDDRIVER_TX_BUFFER_SIZE] - used to store
*  data to sending, modified every function call if TX Software Buffer is used.
*
* Theory:
*  Allows the user to transmit any byte of data in a single transfer.
*
* Side Effects:
*  If this function is called again before the previous byte is finished then
*  the next byte will be appended to the transfer with no time between
*  the byte transfers. Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void LEDDRIVER_WriteTxData(uint16 txData) 
{
    #if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED)

        uint8 tempStatus;
        uint8 tmpTxBufferRead;

        /* Block if TX buffer is FULL: don't overwrite */
        do
        {
            tmpTxBufferRead = LEDDRIVER_txBufferRead;
            if(0u == tmpTxBufferRead)
            {
                tmpTxBufferRead = (LEDDRIVER_TX_BUFFER_SIZE - 1u);
            }
            else
            {
                tmpTxBufferRead--;
            }

        }while(tmpTxBufferRead == LEDDRIVER_txBufferWrite);

        /* Disable TX interrupt to protect global veriables */
        LEDDRIVER_DisableTxInt();

        tempStatus = LEDDRIVER_GET_STATUS_TX(LEDDRIVER_swStatusTx);
        LEDDRIVER_swStatusTx = tempStatus;


        if((LEDDRIVER_txBufferRead == LEDDRIVER_txBufferWrite) &&
           (0u != (LEDDRIVER_swStatusTx & LEDDRIVER_STS_TX_FIFO_NOT_FULL)))
        {
            /* Put data element into the TX FIFO */
            CY_SET_REG16(LEDDRIVER_TXDATA_PTR, txData);
        }
        else
        {
            /* Add to the TX software buffer */
            LEDDRIVER_txBufferWrite++;
            if(LEDDRIVER_txBufferWrite >= LEDDRIVER_TX_BUFFER_SIZE)
            {
                LEDDRIVER_txBufferWrite = 0u;
            }

            if(LEDDRIVER_txBufferWrite == LEDDRIVER_txBufferRead)
            {
                LEDDRIVER_txBufferRead++;
                if(LEDDRIVER_txBufferRead >= LEDDRIVER_TX_BUFFER_SIZE)
                {
                    LEDDRIVER_txBufferRead = 0u;
                }
                LEDDRIVER_txBufferFull = 1u;
            }

            LEDDRIVER_txBuffer[LEDDRIVER_txBufferWrite] = txData;

            LEDDRIVER_TX_STATUS_MASK_REG |= LEDDRIVER_STS_TX_FIFO_NOT_FULL;
        }

        LEDDRIVER_EnableTxInt();

    #else
        /* Wait until TX FIFO has a place */
        while(0u == (LEDDRIVER_TX_STATUS_REG & LEDDRIVER_STS_TX_FIFO_NOT_FULL))
        {
        }

        /* Put data element into the TX FIFO */
        CY_SET_REG16(LEDDRIVER_TXDATA_PTR, txData);

    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */
}


/*******************************************************************************
* Function Name: LEDDRIVER_ReadRxData
********************************************************************************
*
* Summary:
*  Read the next byte of data received across the SPI.
*
* Parameters:
*  None.
*
* Return:
*  The next byte of data read from the FIFO.
*
* Global variables:
*  LEDDRIVER_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  LEDDRIVER_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function
*  call if RX Software Buffer is used.
*  LEDDRIVER_rxBuffer[LEDDRIVER_RX_BUFFER_SIZE] - used to store
*  received data.
*
* Theory:
*  Allows the user to read a byte of data received.
*
* Side Effects:
*  Will return invalid data if the FIFO is empty. The user should Call
*  GetRxBufferSize() and if it returns a non-zero value then it is safe to call
*  ReadByte() function.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint16 LEDDRIVER_ReadRxData(void) 
{
    uint16 rxData;

    #if(LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)

        /* Disable RX interrupt to protect global veriables */
        LEDDRIVER_DisableRxInt();

        if(LEDDRIVER_rxBufferRead != LEDDRIVER_rxBufferWrite)
        {
            if(0u == LEDDRIVER_rxBufferFull)
            {
                LEDDRIVER_rxBufferRead++;
                if(LEDDRIVER_rxBufferRead >= LEDDRIVER_RX_BUFFER_SIZE)
                {
                    LEDDRIVER_rxBufferRead = 0u;
                }
            }
            else
            {
                LEDDRIVER_rxBufferFull = 0u;
            }
        }

        rxData = LEDDRIVER_rxBuffer[LEDDRIVER_rxBufferRead];

        LEDDRIVER_EnableRxInt();

    #else

        rxData = CY_GET_REG16(LEDDRIVER_RXDATA_PTR);

    #endif /* (LEDDRIVER_RX_SOFTWARE_BUF_ENABLED) */

    return(rxData);
}


/*******************************************************************************
* Function Name: LEDDRIVER_GetRxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the RX buffer.
*  If RX Software Buffer not used then function return 0 if FIFO empty or 1 if
*  FIFO not empty. In another case function return size of RX Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the RX buffer.
*
* Global variables:
*  LEDDRIVER_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  LEDDRIVER_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8 LEDDRIVER_GetRxBufferSize(void) 
{
    uint8 size;

    #if(LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)

        /* Disable RX interrupt to protect global veriables */
        LEDDRIVER_DisableRxInt();

        if(LEDDRIVER_rxBufferRead == LEDDRIVER_rxBufferWrite)
        {
            size = 0u;
        }
        else if(LEDDRIVER_rxBufferRead < LEDDRIVER_rxBufferWrite)
        {
            size = (LEDDRIVER_rxBufferWrite - LEDDRIVER_rxBufferRead);
        }
        else
        {
            size = (LEDDRIVER_RX_BUFFER_SIZE - LEDDRIVER_rxBufferRead) + LEDDRIVER_rxBufferWrite;
        }

        LEDDRIVER_EnableRxInt();

    #else

        /* We can only know if there is data in the RX FIFO */
        size = (0u != (LEDDRIVER_RX_STATUS_REG & LEDDRIVER_STS_RX_FIFO_NOT_EMPTY)) ? 1u : 0u;

    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */

    return(size);
}


/*******************************************************************************
* Function Name: LEDDRIVER_GetTxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the TX buffer.
*  If TX Software Buffer not used then function return 0 - if FIFO empty, 1 - if
*  FIFO not full, 4 - if FIFO full. In another case function return size of TX
*  Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the TX buffer.
*
* Global variables:
*  LEDDRIVER_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  LEDDRIVER_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8  LEDDRIVER_GetTxBufferSize(void) 
{
    uint8 size;

    #if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        LEDDRIVER_DisableTxInt();

        if(LEDDRIVER_txBufferRead == LEDDRIVER_txBufferWrite)
        {
            size = 0u;
        }
        else if(LEDDRIVER_txBufferRead < LEDDRIVER_txBufferWrite)
        {
            size = (LEDDRIVER_txBufferWrite - LEDDRIVER_txBufferRead);
        }
        else
        {
            size = (LEDDRIVER_TX_BUFFER_SIZE - LEDDRIVER_txBufferRead) + LEDDRIVER_txBufferWrite;
        }

        LEDDRIVER_EnableTxInt();

    #else

        size = LEDDRIVER_TX_STATUS_REG;

        if(0u != (size & LEDDRIVER_STS_TX_FIFO_EMPTY))
        {
            size = 0u;
        }
        else if(0u != (size & LEDDRIVER_STS_TX_FIFO_NOT_FULL))
        {
            size = 1u;
        }
        else
        {
            size = LEDDRIVER_FIFO_SIZE;
        }

    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */

    return(size);
}


/*******************************************************************************
* Function Name: LEDDRIVER_ClearRxBuffer
********************************************************************************
*
* Summary:
*  Clear the RX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  LEDDRIVER_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer, modified every function
*  call - resets to zero.
*  LEDDRIVER_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any received data not read from the RAM buffer will be lost when overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void LEDDRIVER_ClearRxBuffer(void) 
{
    /* Clear Hardware RX FIFO */
    while(0u !=(LEDDRIVER_RX_STATUS_REG & LEDDRIVER_STS_RX_FIFO_NOT_EMPTY))
    {
        (void) CY_GET_REG16(LEDDRIVER_RXDATA_PTR);
    }

    #if(LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)
        /* Disable RX interrupt to protect global veriables */
        LEDDRIVER_DisableRxInt();

        LEDDRIVER_rxBufferFull  = 0u;
        LEDDRIVER_rxBufferRead  = 0u;
        LEDDRIVER_rxBufferWrite = 0u;

        LEDDRIVER_EnableRxInt();
    #endif /* (LEDDRIVER_RX_SOFTWARE_BUF_ENABLED) */
}


/*******************************************************************************
* Function Name: LEDDRIVER_ClearTxBuffer
********************************************************************************
*
* Summary:
*  Clear the TX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  LEDDRIVER_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call - resets to zero.
*  LEDDRIVER_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any data not yet transmitted from the RAM buffer will be lost when
*  overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void LEDDRIVER_ClearTxBuffer(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    /* Clear TX FIFO */
    LEDDRIVER_AUX_CONTROL_DP0_REG |= ((uint8)  LEDDRIVER_TX_FIFO_CLR);
    LEDDRIVER_AUX_CONTROL_DP0_REG &= ((uint8) ~LEDDRIVER_TX_FIFO_CLR);

    #if(LEDDRIVER_USE_SECOND_DATAPATH)
        /* Clear TX FIFO for 2nd Datapath */
        LEDDRIVER_AUX_CONTROL_DP1_REG |= ((uint8)  LEDDRIVER_TX_FIFO_CLR);
        LEDDRIVER_AUX_CONTROL_DP1_REG &= ((uint8) ~LEDDRIVER_TX_FIFO_CLR);
    #endif /* (LEDDRIVER_USE_SECOND_DATAPATH) */
    CyExitCriticalSection(enableInterrupts);

    #if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        LEDDRIVER_DisableTxInt();

        LEDDRIVER_txBufferFull  = 0u;
        LEDDRIVER_txBufferRead  = 0u;
        LEDDRIVER_txBufferWrite = 0u;

        /* Buffer is EMPTY: disable TX FIFO NOT FULL interrupt */
        LEDDRIVER_TX_STATUS_MASK_REG &= ((uint8) ~LEDDRIVER_STS_TX_FIFO_NOT_FULL);

        LEDDRIVER_EnableTxInt();
    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */
}


#if(0u != LEDDRIVER_BIDIRECTIONAL_MODE)
    /*******************************************************************************
    * Function Name: LEDDRIVER_TxEnable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI master is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to transmit.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void LEDDRIVER_TxEnable(void) 
    {
        LEDDRIVER_CONTROL_REG |= LEDDRIVER_CTRL_TX_SIGNAL_EN;
    }


    /*******************************************************************************
    * Function Name: LEDDRIVER_TxDisable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI master is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to receive.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void LEDDRIVER_TxDisable(void) 
    {
        LEDDRIVER_CONTROL_REG &= ((uint8) ~LEDDRIVER_CTRL_TX_SIGNAL_EN);
    }

#endif /* (0u != LEDDRIVER_BIDIRECTIONAL_MODE) */


/*******************************************************************************
* Function Name: LEDDRIVER_PutArray
********************************************************************************
*
* Summary:
*  Write available data from ROM/RAM to the TX buffer while space is available
*  in the TX buffer. Keep trying until all data is passed to the TX buffer.
*
* Parameters:
*  *buffer: Pointer to the location in RAM containing the data to send
*  byteCount: The number of bytes to move to the transmit buffer.
*
* Return:
*  None.
*
* Side Effects:
*  Will stay in this routine until all data has been sent.  May get locked in
*  this loop if data is not being initiated by the master if there is not
*  enough room in the TX FIFO.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void LEDDRIVER_PutArray(const uint16 buffer[], uint8 byteCount)
                                                                          
{
    uint8 bufIndex;

    bufIndex = 0u;

    while(byteCount > 0u)
    {
        LEDDRIVER_WriteTxData(buffer[bufIndex]);
        bufIndex++;
        byteCount--;
    }
}


/*******************************************************************************
* Function Name: LEDDRIVER_ClearFIFO
********************************************************************************
*
* Summary:
*  Clear the RX and TX FIFO's of all data for a fresh start.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
void LEDDRIVER_ClearFIFO(void) 
{
    uint8 enableInterrupts;

    /* Clear Hardware RX FIFO */
    while(0u !=(LEDDRIVER_RX_STATUS_REG & LEDDRIVER_STS_RX_FIFO_NOT_EMPTY))
    {
        (void) CY_GET_REG16(LEDDRIVER_RXDATA_PTR);
    }

    enableInterrupts = CyEnterCriticalSection();
    /* Clear TX FIFO */
    LEDDRIVER_AUX_CONTROL_DP0_REG |= ((uint8)  LEDDRIVER_TX_FIFO_CLR);
    LEDDRIVER_AUX_CONTROL_DP0_REG &= ((uint8) ~LEDDRIVER_TX_FIFO_CLR);

    #if(LEDDRIVER_USE_SECOND_DATAPATH)
        /* Clear TX FIFO for 2nd Datapath */
        LEDDRIVER_AUX_CONTROL_DP1_REG |= ((uint8)  LEDDRIVER_TX_FIFO_CLR);
        LEDDRIVER_AUX_CONTROL_DP1_REG &= ((uint8) ~LEDDRIVER_TX_FIFO_CLR);
    #endif /* (LEDDRIVER_USE_SECOND_DATAPATH) */
    CyExitCriticalSection(enableInterrupts);
}


/* Following functions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects.
*/


/*******************************************************************************
* Function Name: LEDDRIVER_EnableInt
********************************************************************************
*
* Summary:
*  Enable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void LEDDRIVER_EnableInt(void) 
{
    LEDDRIVER_EnableRxInt();
    LEDDRIVER_EnableTxInt();
}


/*******************************************************************************
* Function Name: LEDDRIVER_DisableInt
********************************************************************************
*
* Summary:
*  Disable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void LEDDRIVER_DisableInt(void) 
{
    LEDDRIVER_DisableTxInt();
    LEDDRIVER_DisableRxInt();
}


/*******************************************************************************
* Function Name: LEDDRIVER_SetInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void LEDDRIVER_SetInterruptMode(uint8 intSrc) 
{
    LEDDRIVER_TX_STATUS_MASK_REG  = (intSrc & ((uint8) ~LEDDRIVER_STS_SPI_IDLE));
    LEDDRIVER_RX_STATUS_MASK_REG  =  intSrc;
}


/*******************************************************************************
* Function Name: LEDDRIVER_ReadStatus
********************************************************************************
*
* Summary:
*  Read the status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the status register.
*
* Global variables:
*  LEDDRIVER_swStatus - used to store in software status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the status register for error detection
*  and flow control.
*
* Side Effects:
*  Clear status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 LEDDRIVER_ReadStatus(void) 
{
    uint8 tmpStatus;

    #if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED || LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)

        LEDDRIVER_DisableInt();

        tmpStatus  = LEDDRIVER_GET_STATUS_RX(LEDDRIVER_swStatusRx);
        tmpStatus |= LEDDRIVER_GET_STATUS_TX(LEDDRIVER_swStatusTx);
        tmpStatus &= ((uint8) ~LEDDRIVER_STS_SPI_IDLE);

        LEDDRIVER_swStatusTx = 0u;
        LEDDRIVER_swStatusRx = 0u;

        LEDDRIVER_EnableInt();

    #else

        tmpStatus  = LEDDRIVER_RX_STATUS_REG;
        tmpStatus |= LEDDRIVER_TX_STATUS_REG;
        tmpStatus &= ((uint8) ~LEDDRIVER_STS_SPI_IDLE);

    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED || LEDDRIVER_RX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/* [] END OF FILE */
