/*******************************************************************************
* File Name: LEDDRIVER.h
* Version 2.50
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Master Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIM_LEDDRIVER_H)
#define CY_SPIM_LEDDRIVER_H

#include "cyfitter.h"
#include "cytypes.h"
#include "CyLib.h" /* For CyEnterCriticalSection() and CyExitCriticalSection() functions */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define LEDDRIVER_INTERNAL_CLOCK             (0u)

#if(0u != LEDDRIVER_INTERNAL_CLOCK)
    #include "LEDDRIVER_IntClock.h"
#endif /* (0u != LEDDRIVER_INTERNAL_CLOCK) */

#define LEDDRIVER_MODE                       (1u)
#define LEDDRIVER_DATA_WIDTH                 (16u)
#define LEDDRIVER_MODE_USE_ZERO              (1u)
#define LEDDRIVER_BIDIRECTIONAL_MODE         (0u)

/* Internal interrupt handling */
#define LEDDRIVER_TX_BUFFER_SIZE             (4u)
#define LEDDRIVER_RX_BUFFER_SIZE             (4u)
#define LEDDRIVER_INTERNAL_TX_INT_ENABLED    (0u)
#define LEDDRIVER_INTERNAL_RX_INT_ENABLED    (0u)

#define LEDDRIVER_SINGLE_REG_SIZE            (8u)
#define LEDDRIVER_USE_SECOND_DATAPATH        (LEDDRIVER_DATA_WIDTH > LEDDRIVER_SINGLE_REG_SIZE)

#define LEDDRIVER_FIFO_SIZE                  (4u)
#define LEDDRIVER_TX_SOFTWARE_BUF_ENABLED    ((0u != LEDDRIVER_INTERNAL_TX_INT_ENABLED) && \
                                                     (LEDDRIVER_TX_BUFFER_SIZE > LEDDRIVER_FIFO_SIZE))

#define LEDDRIVER_RX_SOFTWARE_BUF_ENABLED    ((0u != LEDDRIVER_INTERNAL_RX_INT_ENABLED) && \
                                                     (LEDDRIVER_RX_BUFFER_SIZE > LEDDRIVER_FIFO_SIZE))


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
    uint8 cntrPeriod;
} LEDDRIVER_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  LEDDRIVER_Init(void)                           ;
void  LEDDRIVER_Enable(void)                         ;
void  LEDDRIVER_Start(void)                          ;
void  LEDDRIVER_Stop(void)                           ;

void  LEDDRIVER_EnableTxInt(void)                    ;
void  LEDDRIVER_EnableRxInt(void)                    ;
void  LEDDRIVER_DisableTxInt(void)                   ;
void  LEDDRIVER_DisableRxInt(void)                   ;

void  LEDDRIVER_Sleep(void)                          ;
void  LEDDRIVER_Wakeup(void)                         ;
void  LEDDRIVER_SaveConfig(void)                     ;
void  LEDDRIVER_RestoreConfig(void)                  ;

void  LEDDRIVER_SetTxInterruptMode(uint8 intSrc)     ;
void  LEDDRIVER_SetRxInterruptMode(uint8 intSrc)     ;
uint8 LEDDRIVER_ReadTxStatus(void)                   ;
uint8 LEDDRIVER_ReadRxStatus(void)                   ;
void  LEDDRIVER_WriteTxData(uint16 txData)  \
                                                            ;
uint16 LEDDRIVER_ReadRxData(void) \
                                                            ;
uint8 LEDDRIVER_GetRxBufferSize(void)                ;
uint8 LEDDRIVER_GetTxBufferSize(void)                ;
void  LEDDRIVER_ClearRxBuffer(void)                  ;
void  LEDDRIVER_ClearTxBuffer(void)                  ;
void  LEDDRIVER_ClearFIFO(void)                              ;
void  LEDDRIVER_PutArray(const uint16 buffer[], uint8 byteCount) \
                                                            ;

#if(0u != LEDDRIVER_BIDIRECTIONAL_MODE)
    void  LEDDRIVER_TxEnable(void)                   ;
    void  LEDDRIVER_TxDisable(void)                  ;
#endif /* (0u != LEDDRIVER_BIDIRECTIONAL_MODE) */

CY_ISR_PROTO(LEDDRIVER_TX_ISR);
CY_ISR_PROTO(LEDDRIVER_RX_ISR);


/***************************************
*   Variable with external linkage
***************************************/

extern uint8 LEDDRIVER_initVar;


/***************************************
*           API Constants
***************************************/

#define LEDDRIVER_TX_ISR_NUMBER     ((uint8) (LEDDRIVER_TxInternalInterrupt__INTC_NUMBER))
#define LEDDRIVER_RX_ISR_NUMBER     ((uint8) (LEDDRIVER_RxInternalInterrupt__INTC_NUMBER))

#define LEDDRIVER_TX_ISR_PRIORITY   ((uint8) (LEDDRIVER_TxInternalInterrupt__INTC_PRIOR_NUM))
#define LEDDRIVER_RX_ISR_PRIORITY   ((uint8) (LEDDRIVER_RxInternalInterrupt__INTC_PRIOR_NUM))


/***************************************
*    Initial Parameter Constants
***************************************/

#define LEDDRIVER_INT_ON_SPI_DONE    ((uint8) (0u   << LEDDRIVER_STS_SPI_DONE_SHIFT))
#define LEDDRIVER_INT_ON_TX_EMPTY    ((uint8) (0u   << LEDDRIVER_STS_TX_FIFO_EMPTY_SHIFT))
#define LEDDRIVER_INT_ON_TX_NOT_FULL ((uint8) (0u << \
                                                                           LEDDRIVER_STS_TX_FIFO_NOT_FULL_SHIFT))
#define LEDDRIVER_INT_ON_BYTE_COMP   ((uint8) (0u  << LEDDRIVER_STS_BYTE_COMPLETE_SHIFT))
#define LEDDRIVER_INT_ON_SPI_IDLE    ((uint8) (0u   << LEDDRIVER_STS_SPI_IDLE_SHIFT))

/* Disable TX_NOT_FULL if software buffer is used */
#define LEDDRIVER_INT_ON_TX_NOT_FULL_DEF ((LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) ? \
                                                                        (0u) : (LEDDRIVER_INT_ON_TX_NOT_FULL))

/* TX interrupt mask */
#define LEDDRIVER_TX_INIT_INTERRUPTS_MASK    (LEDDRIVER_INT_ON_SPI_DONE  | \
                                                     LEDDRIVER_INT_ON_TX_EMPTY  | \
                                                     LEDDRIVER_INT_ON_TX_NOT_FULL_DEF | \
                                                     LEDDRIVER_INT_ON_BYTE_COMP | \
                                                     LEDDRIVER_INT_ON_SPI_IDLE)

#define LEDDRIVER_INT_ON_RX_FULL         ((uint8) (0u << \
                                                                          LEDDRIVER_STS_RX_FIFO_FULL_SHIFT))
#define LEDDRIVER_INT_ON_RX_NOT_EMPTY    ((uint8) (0u << \
                                                                          LEDDRIVER_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define LEDDRIVER_INT_ON_RX_OVER         ((uint8) (0u << \
                                                                          LEDDRIVER_STS_RX_FIFO_OVERRUN_SHIFT))

/* RX interrupt mask */
#define LEDDRIVER_RX_INIT_INTERRUPTS_MASK    (LEDDRIVER_INT_ON_RX_FULL      | \
                                                     LEDDRIVER_INT_ON_RX_NOT_EMPTY | \
                                                     LEDDRIVER_INT_ON_RX_OVER)
/* Nubmer of bits to receive/transmit */
#define LEDDRIVER_BITCTR_INIT            (((uint8) (LEDDRIVER_DATA_WIDTH << 1u)) - 1u)


/***************************************
*             Registers
***************************************/
#if(CY_PSOC3 || CY_PSOC5)
    #define LEDDRIVER_TXDATA_REG (* (reg16 *) \
                                                LEDDRIVER_BSPIM_sR16_Dp_u0__F0_REG)
    #define LEDDRIVER_TXDATA_PTR (  (reg16 *) \
                                                LEDDRIVER_BSPIM_sR16_Dp_u0__F0_REG)
    #define LEDDRIVER_RXDATA_REG (* (reg16 *) \
                                                LEDDRIVER_BSPIM_sR16_Dp_u0__F1_REG)
    #define LEDDRIVER_RXDATA_PTR (  (reg16 *) \
                                                LEDDRIVER_BSPIM_sR16_Dp_u0__F1_REG)
#else   /* PSOC4 */
    #if(LEDDRIVER_USE_SECOND_DATAPATH)
        #define LEDDRIVER_TXDATA_REG (* (reg16 *) \
                                          LEDDRIVER_BSPIM_sR16_Dp_u0__16BIT_F0_REG)
        #define LEDDRIVER_TXDATA_PTR (  (reg16 *) \
                                          LEDDRIVER_BSPIM_sR16_Dp_u0__16BIT_F0_REG)
        #define LEDDRIVER_RXDATA_REG (* (reg16 *) \
                                          LEDDRIVER_BSPIM_sR16_Dp_u0__16BIT_F1_REG)
        #define LEDDRIVER_RXDATA_PTR (  (reg16 *) \
                                          LEDDRIVER_BSPIM_sR16_Dp_u0__16BIT_F1_REG)
    #else
        #define LEDDRIVER_TXDATA_REG (* (reg8 *) \
                                                LEDDRIVER_BSPIM_sR16_Dp_u0__F0_REG)
        #define LEDDRIVER_TXDATA_PTR (  (reg8 *) \
                                                LEDDRIVER_BSPIM_sR16_Dp_u0__F0_REG)
        #define LEDDRIVER_RXDATA_REG (* (reg8 *) \
                                                LEDDRIVER_BSPIM_sR16_Dp_u0__F1_REG)
        #define LEDDRIVER_RXDATA_PTR (  (reg8 *) \
                                                LEDDRIVER_BSPIM_sR16_Dp_u0__F1_REG)
    #endif /* (LEDDRIVER_USE_SECOND_DATAPATH) */
#endif     /* (CY_PSOC3 || CY_PSOC5) */

#define LEDDRIVER_AUX_CONTROL_DP0_REG (* (reg8 *) \
                                        LEDDRIVER_BSPIM_sR16_Dp_u0__DP_AUX_CTL_REG)
#define LEDDRIVER_AUX_CONTROL_DP0_PTR (  (reg8 *) \
                                        LEDDRIVER_BSPIM_sR16_Dp_u0__DP_AUX_CTL_REG)

#if(LEDDRIVER_USE_SECOND_DATAPATH)
    #define LEDDRIVER_AUX_CONTROL_DP1_REG  (* (reg8 *) \
                                        LEDDRIVER_BSPIM_sR16_Dp_u1__DP_AUX_CTL_REG)
    #define LEDDRIVER_AUX_CONTROL_DP1_PTR  (  (reg8 *) \
                                        LEDDRIVER_BSPIM_sR16_Dp_u1__DP_AUX_CTL_REG)
#endif /* (LEDDRIVER_USE_SECOND_DATAPATH) */

#define LEDDRIVER_COUNTER_PERIOD_REG     (* (reg8 *) LEDDRIVER_BSPIM_BitCounter__PERIOD_REG)
#define LEDDRIVER_COUNTER_PERIOD_PTR     (  (reg8 *) LEDDRIVER_BSPIM_BitCounter__PERIOD_REG)
#define LEDDRIVER_COUNTER_CONTROL_REG    (* (reg8 *) LEDDRIVER_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)
#define LEDDRIVER_COUNTER_CONTROL_PTR    (  (reg8 *) LEDDRIVER_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)

#define LEDDRIVER_TX_STATUS_REG          (* (reg8 *) LEDDRIVER_BSPIM_TxStsReg__STATUS_REG)
#define LEDDRIVER_TX_STATUS_PTR          (  (reg8 *) LEDDRIVER_BSPIM_TxStsReg__STATUS_REG)
#define LEDDRIVER_RX_STATUS_REG          (* (reg8 *) LEDDRIVER_BSPIM_RxStsReg__STATUS_REG)
#define LEDDRIVER_RX_STATUS_PTR          (  (reg8 *) LEDDRIVER_BSPIM_RxStsReg__STATUS_REG)

#define LEDDRIVER_CONTROL_REG            (* (reg8 *) \
                                      LEDDRIVER_BSPIM_BidirMode_CtrlReg__CONTROL_REG)
#define LEDDRIVER_CONTROL_PTR            (  (reg8 *) \
                                      LEDDRIVER_BSPIM_BidirMode_CtrlReg__CONTROL_REG)

#define LEDDRIVER_TX_STATUS_MASK_REG     (* (reg8 *) LEDDRIVER_BSPIM_TxStsReg__MASK_REG)
#define LEDDRIVER_TX_STATUS_MASK_PTR     (  (reg8 *) LEDDRIVER_BSPIM_TxStsReg__MASK_REG)
#define LEDDRIVER_RX_STATUS_MASK_REG     (* (reg8 *) LEDDRIVER_BSPIM_RxStsReg__MASK_REG)
#define LEDDRIVER_RX_STATUS_MASK_PTR     (  (reg8 *) LEDDRIVER_BSPIM_RxStsReg__MASK_REG)

#define LEDDRIVER_TX_STATUS_ACTL_REG     (* (reg8 *) LEDDRIVER_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define LEDDRIVER_TX_STATUS_ACTL_PTR     (  (reg8 *) LEDDRIVER_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define LEDDRIVER_RX_STATUS_ACTL_REG     (* (reg8 *) LEDDRIVER_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)
#define LEDDRIVER_RX_STATUS_ACTL_PTR     (  (reg8 *) LEDDRIVER_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)

#if(LEDDRIVER_USE_SECOND_DATAPATH)
    #define LEDDRIVER_AUX_CONTROLDP1     (LEDDRIVER_AUX_CONTROL_DP1_REG)
#endif /* (LEDDRIVER_USE_SECOND_DATAPATH) */


/***************************************
*       Register Constants
***************************************/

/* Status Register Definitions */
#define LEDDRIVER_STS_SPI_DONE_SHIFT             (0x00u)
#define LEDDRIVER_STS_TX_FIFO_EMPTY_SHIFT        (0x01u)
#define LEDDRIVER_STS_TX_FIFO_NOT_FULL_SHIFT     (0x02u)
#define LEDDRIVER_STS_BYTE_COMPLETE_SHIFT        (0x03u)
#define LEDDRIVER_STS_SPI_IDLE_SHIFT             (0x04u)
#define LEDDRIVER_STS_RX_FIFO_FULL_SHIFT         (0x04u)
#define LEDDRIVER_STS_RX_FIFO_NOT_EMPTY_SHIFT    (0x05u)
#define LEDDRIVER_STS_RX_FIFO_OVERRUN_SHIFT      (0x06u)

#define LEDDRIVER_STS_SPI_DONE           ((uint8) (0x01u << LEDDRIVER_STS_SPI_DONE_SHIFT))
#define LEDDRIVER_STS_TX_FIFO_EMPTY      ((uint8) (0x01u << LEDDRIVER_STS_TX_FIFO_EMPTY_SHIFT))
#define LEDDRIVER_STS_TX_FIFO_NOT_FULL   ((uint8) (0x01u << LEDDRIVER_STS_TX_FIFO_NOT_FULL_SHIFT))
#define LEDDRIVER_STS_BYTE_COMPLETE      ((uint8) (0x01u << LEDDRIVER_STS_BYTE_COMPLETE_SHIFT))
#define LEDDRIVER_STS_SPI_IDLE           ((uint8) (0x01u << LEDDRIVER_STS_SPI_IDLE_SHIFT))
#define LEDDRIVER_STS_RX_FIFO_FULL       ((uint8) (0x01u << LEDDRIVER_STS_RX_FIFO_FULL_SHIFT))
#define LEDDRIVER_STS_RX_FIFO_NOT_EMPTY  ((uint8) (0x01u << LEDDRIVER_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define LEDDRIVER_STS_RX_FIFO_OVERRUN    ((uint8) (0x01u << LEDDRIVER_STS_RX_FIFO_OVERRUN_SHIFT))

/* TX and RX masks for clear on read bits */
#define LEDDRIVER_TX_STS_CLR_ON_RD_BYTES_MASK    (0x09u)
#define LEDDRIVER_RX_STS_CLR_ON_RD_BYTES_MASK    (0x40u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define LEDDRIVER_INT_ENABLE     (0x10u) /* Enable interrupt from statusi */
#define LEDDRIVER_TX_FIFO_CLR    (0x01u) /* F0 - TX FIFO */
#define LEDDRIVER_RX_FIFO_CLR    (0x02u) /* F1 - RX FIFO */
#define LEDDRIVER_FIFO_CLR       (LEDDRIVER_TX_FIFO_CLR | LEDDRIVER_RX_FIFO_CLR)

/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define LEDDRIVER_CNTR_ENABLE    (0x20u) /* Enable CNT7 */

/* Bi-Directional mode control bit */
#define LEDDRIVER_CTRL_TX_SIGNAL_EN  (0x01u)

/* Datapath Auxillary Control Register definitions */
#define LEDDRIVER_AUX_CTRL_FIFO0_CLR         (0x01u)
#define LEDDRIVER_AUX_CTRL_FIFO1_CLR         (0x02u)
#define LEDDRIVER_AUX_CTRL_FIFO0_LVL         (0x04u)
#define LEDDRIVER_AUX_CTRL_FIFO1_LVL         (0x08u)
#define LEDDRIVER_STATUS_ACTL_INT_EN_MASK    (0x10u)

/* Component disabled */
#define LEDDRIVER_DISABLED   (0u)


/***************************************
*       Macros
***************************************/

/* Returns true if componentn enabled */
#define LEDDRIVER_IS_ENABLED (0u != (LEDDRIVER_TX_STATUS_ACTL_REG & LEDDRIVER_INT_ENABLE))

/* Retuns TX status register */
#define LEDDRIVER_GET_STATUS_TX(swTxSts) ( (uint8)(LEDDRIVER_TX_STATUS_REG | \
                                                          ((swTxSts) & LEDDRIVER_TX_STS_CLR_ON_RD_BYTES_MASK)) )
/* Retuns RX status register */
#define LEDDRIVER_GET_STATUS_RX(swRxSts) ( (uint8)(LEDDRIVER_RX_STATUS_REG | \
                                                          ((swRxSts) & LEDDRIVER_RX_STS_CLR_ON_RD_BYTES_MASK)) )


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define LEDDRIVER_WriteByte   LEDDRIVER_WriteTxData
#define LEDDRIVER_ReadByte    LEDDRIVER_ReadRxData
void  LEDDRIVER_SetInterruptMode(uint8 intSrc)       ;
uint8 LEDDRIVER_ReadStatus(void)                     ;
void  LEDDRIVER_EnableInt(void)                      ;
void  LEDDRIVER_DisableInt(void)                     ;

#define LEDDRIVER_TXDATA                 (LEDDRIVER_TXDATA_REG)
#define LEDDRIVER_RXDATA                 (LEDDRIVER_RXDATA_REG)
#define LEDDRIVER_AUX_CONTROLDP0         (LEDDRIVER_AUX_CONTROL_DP0_REG)
#define LEDDRIVER_TXBUFFERREAD           (LEDDRIVER_txBufferRead)
#define LEDDRIVER_TXBUFFERWRITE          (LEDDRIVER_txBufferWrite)
#define LEDDRIVER_RXBUFFERREAD           (LEDDRIVER_rxBufferRead)
#define LEDDRIVER_RXBUFFERWRITE          (LEDDRIVER_rxBufferWrite)

#define LEDDRIVER_COUNTER_PERIOD         (LEDDRIVER_COUNTER_PERIOD_REG)
#define LEDDRIVER_COUNTER_CONTROL        (LEDDRIVER_COUNTER_CONTROL_REG)
#define LEDDRIVER_STATUS                 (LEDDRIVER_TX_STATUS_REG)
#define LEDDRIVER_CONTROL                (LEDDRIVER_CONTROL_REG)
#define LEDDRIVER_STATUS_MASK            (LEDDRIVER_TX_STATUS_MASK_REG)
#define LEDDRIVER_STATUS_ACTL            (LEDDRIVER_TX_STATUS_ACTL_REG)

#define LEDDRIVER_INIT_INTERRUPTS_MASK  (LEDDRIVER_INT_ON_SPI_DONE     | \
                                                LEDDRIVER_INT_ON_TX_EMPTY     | \
                                                LEDDRIVER_INT_ON_TX_NOT_FULL_DEF  | \
                                                LEDDRIVER_INT_ON_RX_FULL      | \
                                                LEDDRIVER_INT_ON_RX_NOT_EMPTY | \
                                                LEDDRIVER_INT_ON_RX_OVER      | \
                                                LEDDRIVER_INT_ON_BYTE_COMP)
                                                
#define LEDDRIVER_DataWidth                  (LEDDRIVER_DATA_WIDTH)
#define LEDDRIVER_InternalClockUsed          (LEDDRIVER_INTERNAL_CLOCK)
#define LEDDRIVER_InternalTxInterruptEnabled (LEDDRIVER_INTERNAL_TX_INT_ENABLED)
#define LEDDRIVER_InternalRxInterruptEnabled (LEDDRIVER_INTERNAL_RX_INT_ENABLED)
#define LEDDRIVER_ModeUseZero                (LEDDRIVER_MODE_USE_ZERO)
#define LEDDRIVER_BidirectionalMode          (LEDDRIVER_BIDIRECTIONAL_MODE)
#define LEDDRIVER_Mode                       (LEDDRIVER_MODE)
#define LEDDRIVER_DATAWIDHT                  (LEDDRIVER_DATA_WIDTH)
#define LEDDRIVER_InternalInterruptEnabled   (0u)

#define LEDDRIVER_TXBUFFERSIZE   (LEDDRIVER_TX_BUFFER_SIZE)
#define LEDDRIVER_RXBUFFERSIZE   (LEDDRIVER_RX_BUFFER_SIZE)

#define LEDDRIVER_TXBUFFER       LEDDRIVER_txBuffer
#define LEDDRIVER_RXBUFFER       LEDDRIVER_rxBuffer

#endif /* (CY_SPIM_LEDDRIVER_H) */


/* [] END OF FILE */
