/*******************************************************************************
* File Name: LEDDRIVER_INT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "LEDDRIVER_PVT.h"
#include "cyapicallbacks.h"

/* User code required at start of ISR */
/* `#START LEDDRIVER_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: LEDDRIVER_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  LEDDRIVER_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  LEDDRIVER_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  LEDDRIVER_txBuffer[LEDDRIVER_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(LEDDRIVER_TX_ISR)
{
    #if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef LEDDRIVER_TX_ISR_ENTRY_CALLBACK
        LEDDRIVER_TX_ISR_EntryCallback();
    #endif /* LEDDRIVER_TX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START LEDDRIVER_TX_ISR_START` */

    /* `#END` */
    
    #if(LEDDRIVER_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(LEDDRIVER_txBufferRead != LEDDRIVER_txBufferWrite)
        {
            tmpStatus = LEDDRIVER_GET_STATUS_TX(LEDDRIVER_swStatusTx);
            LEDDRIVER_swStatusTx = tmpStatus;

            if(0u != (LEDDRIVER_swStatusTx & LEDDRIVER_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == LEDDRIVER_txBufferFull)
                {
                   LEDDRIVER_txBufferRead++;

                    if(LEDDRIVER_txBufferRead >= LEDDRIVER_TX_BUFFER_SIZE)
                    {
                        LEDDRIVER_txBufferRead = 0u;
                    }
                }
                else
                {
                    LEDDRIVER_txBufferFull = 0u;
                }

                /* Put data element into the TX FIFO */
                CY_SET_REG16(LEDDRIVER_TXDATA_PTR, 
                                             LEDDRIVER_txBuffer[LEDDRIVER_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(LEDDRIVER_txBufferRead == LEDDRIVER_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            LEDDRIVER_TX_STATUS_MASK_REG &= ((uint8) ~LEDDRIVER_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (LEDDRIVER_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START LEDDRIVER_TX_ISR_END` */

    /* `#END` */
    
    #ifdef LEDDRIVER_TX_ISR_EXIT_CALLBACK
        LEDDRIVER_TX_ISR_ExitCallback();
    #endif /* LEDDRIVER_TX_ISR_EXIT_CALLBACK */
}


/*******************************************************************************
* Function Name: LEDDRIVER_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  LEDDRIVER_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  LEDDRIVER_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  LEDDRIVER_rxBuffer[LEDDRIVER_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(LEDDRIVER_RX_ISR)
{
    #if(LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint16 rxData;
    #endif /* (LEDDRIVER_RX_SOFTWARE_BUF_ENABLED) */

    #ifdef LEDDRIVER_RX_ISR_ENTRY_CALLBACK
        LEDDRIVER_RX_ISR_EntryCallback();
    #endif /* LEDDRIVER_RX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START LEDDRIVER_RX_ISR_START` */

    /* `#END` */
    
    #if(LEDDRIVER_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = LEDDRIVER_GET_STATUS_RX(LEDDRIVER_swStatusRx);
        LEDDRIVER_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (LEDDRIVER_swStatusRx & LEDDRIVER_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG16(LEDDRIVER_RXDATA_PTR);

            /* Set next pointer. */
            LEDDRIVER_rxBufferWrite++;
            if(LEDDRIVER_rxBufferWrite >= LEDDRIVER_RX_BUFFER_SIZE)
            {
                LEDDRIVER_rxBufferWrite = 0u;
            }

            if(LEDDRIVER_rxBufferWrite == LEDDRIVER_rxBufferRead)
            {
                LEDDRIVER_rxBufferRead++;
                if(LEDDRIVER_rxBufferRead >= LEDDRIVER_RX_BUFFER_SIZE)
                {
                    LEDDRIVER_rxBufferRead = 0u;
                }

                LEDDRIVER_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            LEDDRIVER_rxBuffer[LEDDRIVER_rxBufferWrite] = rxData;

            tmpStatus = LEDDRIVER_GET_STATUS_RX(LEDDRIVER_swStatusRx);
            LEDDRIVER_swStatusRx = tmpStatus;
        }

    #endif /* (LEDDRIVER_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START LEDDRIVER_RX_ISR_END` */

    /* `#END` */
    
    #ifdef LEDDRIVER_RX_ISR_EXIT_CALLBACK
        LEDDRIVER_RX_ISR_ExitCallback();
    #endif /* LEDDRIVER_RX_ISR_EXIT_CALLBACK */
}

/* [] END OF FILE */
