/*******************************************************************************
* File Name: TempI2C_INT.c
* Version 3.50
*
* Description:
*  This file provides the source code of Interrupt Service Routine (ISR)
*  for the I2C component.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "TempI2C_PVT.h"
#include "cyapicallbacks.h"


/*******************************************************************************
*  Place your includes, defines and code here.
********************************************************************************/
/* `#START TempI2C_ISR_intc` */

/* `#END` */


/*******************************************************************************
* Function Name: TempI2C_ISR
********************************************************************************
*
* Summary:
*  The handler for the I2C interrupt. The slave and master operations are
*  handled here.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Reentrant:
*  No.
*
*******************************************************************************/
CY_ISR(TempI2C_ISR)
{
#if (TempI2C_MODE_SLAVE_ENABLED)
   uint8  tmp8;
#endif  /* (TempI2C_MODE_SLAVE_ENABLED) */

    uint8  tmpCsr;
    
#ifdef TempI2C_ISR_ENTRY_CALLBACK
    TempI2C_ISR_EntryCallback();
#endif /* TempI2C_ISR_ENTRY_CALLBACK */
    

#if(TempI2C_TIMEOUT_FF_ENABLED)
    if(0u != TempI2C_TimeoutGetStatus())
    {
        TempI2C_TimeoutReset();
        TempI2C_state = TempI2C_SM_EXIT_IDLE;
        /* TempI2C_CSR_REG should be cleared after reset */
    }
#endif /* (TempI2C_TIMEOUT_FF_ENABLED) */


    tmpCsr = TempI2C_CSR_REG;      /* Make copy as interrupts clear */

#if(TempI2C_MODE_MULTI_MASTER_SLAVE_ENABLED)
    if(TempI2C_CHECK_START_GEN(TempI2C_MCSR_REG))
    {
        TempI2C_CLEAR_START_GEN;

        /* Set transfer complete and error flags */
        TempI2C_mstrStatus |= (TempI2C_MSTAT_ERR_XFER |
                                        TempI2C_GET_MSTAT_CMPLT);

        /* Slave was addressed */
        TempI2C_state = TempI2C_SM_SLAVE;
    }
#endif /* (TempI2C_MODE_MULTI_MASTER_SLAVE_ENABLED) */


#if(TempI2C_MODE_MULTI_MASTER_ENABLED)
    if(TempI2C_CHECK_LOST_ARB(tmpCsr))
    {
        /* Set errors */
        TempI2C_mstrStatus |= (TempI2C_MSTAT_ERR_XFER     |
                                        TempI2C_MSTAT_ERR_ARB_LOST |
                                        TempI2C_GET_MSTAT_CMPLT);

        TempI2C_DISABLE_INT_ON_STOP; /* Interrupt on Stop is enabled by write */

        #if(TempI2C_MODE_MULTI_MASTER_SLAVE_ENABLED)
            if(TempI2C_CHECK_ADDRESS_STS(tmpCsr))
            {
                /* Slave was addressed */
                TempI2C_state = TempI2C_SM_SLAVE;
            }
            else
            {
                TempI2C_BUS_RELEASE;

                TempI2C_state = TempI2C_SM_EXIT_IDLE;
            }
        #else
            TempI2C_BUS_RELEASE;

            TempI2C_state = TempI2C_SM_EXIT_IDLE;

        #endif /* (TempI2C_MODE_MULTI_MASTER_SLAVE_ENABLED) */
    }
#endif /* (TempI2C_MODE_MULTI_MASTER_ENABLED) */

    /* Check for master operation mode */
    if(TempI2C_CHECK_SM_MASTER)
    {
    #if(TempI2C_MODE_MASTER_ENABLED)
        if(TempI2C_CHECK_BYTE_COMPLETE(tmpCsr))
        {
            switch (TempI2C_state)
            {
            case TempI2C_SM_MSTR_WR_ADDR:  /* After address is sent, write data */
            case TempI2C_SM_MSTR_RD_ADDR:  /* After address is sent, read data */

                tmpCsr &= ((uint8) ~TempI2C_CSR_STOP_STATUS); /* Clear Stop bit history on address phase */

                if(TempI2C_CHECK_ADDR_ACK(tmpCsr))
                {
                    /* Setup for transmit or receive of data */
                    if(TempI2C_state == TempI2C_SM_MSTR_WR_ADDR)   /* TRANSMIT data */
                    {
                        /* Check if at least one byte to transfer */
                        if(TempI2C_mstrWrBufSize > 0u)
                        {
                            /* Load the 1st data byte */
                            TempI2C_DATA_REG = TempI2C_mstrWrBufPtr[0u];
                            TempI2C_TRANSMIT_DATA;
                            TempI2C_mstrWrBufIndex = 1u;   /* Set index to 2nd element */

                            /* Set transmit state until done */
                            TempI2C_state = TempI2C_SM_MSTR_WR_DATA;
                        }
                        /* End of buffer: complete writing */
                        else if(TempI2C_CHECK_NO_STOP(TempI2C_mstrControl))
                        {
                            /* Set write complete and master halted */
                            TempI2C_mstrStatus |= (TempI2C_MSTAT_XFER_HALT |
                                                            TempI2C_MSTAT_WR_CMPLT);

                            TempI2C_state = TempI2C_SM_MSTR_HALT; /* Expect ReStart */
                            TempI2C_DisableInt();
                        }
                        else
                        {
                            TempI2C_ENABLE_INT_ON_STOP; /* Enable interrupt on Stop, to catch it */
                            TempI2C_GENERATE_STOP;
                        }
                    }
                    else  /* Master receive data */
                    {
                        TempI2C_READY_TO_READ; /* Release bus to read data */

                        TempI2C_state  = TempI2C_SM_MSTR_RD_DATA;
                    }
                }
                /* Address is NACKed */
                else if(TempI2C_CHECK_ADDR_NAK(tmpCsr))
                {
                    /* Set Address NAK error */
                    TempI2C_mstrStatus |= (TempI2C_MSTAT_ERR_XFER |
                                                    TempI2C_MSTAT_ERR_ADDR_NAK);

                    if(TempI2C_CHECK_NO_STOP(TempI2C_mstrControl))
                    {
                        TempI2C_mstrStatus |= (TempI2C_MSTAT_XFER_HALT |
                                                        TempI2C_GET_MSTAT_CMPLT);

                        TempI2C_state = TempI2C_SM_MSTR_HALT; /* Expect RESTART */
                        TempI2C_DisableInt();
                    }
                    else  /* Do normal Stop */
                    {
                        TempI2C_ENABLE_INT_ON_STOP; /* Enable interrupt on Stop, to catch it */
                        TempI2C_GENERATE_STOP;
                    }
                }
                else
                {
                    /* Address phase is not set for some reason: error */
                    #if(TempI2C_TIMEOUT_ENABLED)
                        /* Exit interrupt to take chance for timeout timer to handle this case */
                        TempI2C_DisableInt();
                        TempI2C_ClearPendingInt();
                    #else
                        /* Block execution flow: unexpected condition */
                        CYASSERT(0u != 0u);
                    #endif /* (TempI2C_TIMEOUT_ENABLED) */
                }
                break;

            case TempI2C_SM_MSTR_WR_DATA:

                if(TempI2C_CHECK_DATA_ACK(tmpCsr))
                {
                    /* Check if end of buffer */
                    if(TempI2C_mstrWrBufIndex  < TempI2C_mstrWrBufSize)
                    {
                        TempI2C_DATA_REG =
                                                 TempI2C_mstrWrBufPtr[TempI2C_mstrWrBufIndex];
                        TempI2C_TRANSMIT_DATA;
                        TempI2C_mstrWrBufIndex++;
                    }
                    /* End of buffer: complete writing */
                    else if(TempI2C_CHECK_NO_STOP(TempI2C_mstrControl))
                    {
                        /* Set write complete and master halted */
                        TempI2C_mstrStatus |= (TempI2C_MSTAT_XFER_HALT |
                                                        TempI2C_MSTAT_WR_CMPLT);

                        TempI2C_state = TempI2C_SM_MSTR_HALT;    /* Expect restart */
                        TempI2C_DisableInt();
                    }
                    else  /* Do normal Stop */
                    {
                        TempI2C_ENABLE_INT_ON_STOP;    /* Enable interrupt on Stop, to catch it */
                        TempI2C_GENERATE_STOP;
                    }
                }
                /* Last byte NAKed: end writing */
                else if(TempI2C_CHECK_NO_STOP(TempI2C_mstrControl))
                {
                    /* Set write complete, short transfer and master halted */
                    TempI2C_mstrStatus |= (TempI2C_MSTAT_ERR_XFER       |
                                                    TempI2C_MSTAT_ERR_SHORT_XFER |
                                                    TempI2C_MSTAT_XFER_HALT      |
                                                    TempI2C_MSTAT_WR_CMPLT);

                    TempI2C_state = TempI2C_SM_MSTR_HALT;    /* Expect ReStart */
                    TempI2C_DisableInt();
                }
                else  /* Do normal Stop */
                {
                    TempI2C_ENABLE_INT_ON_STOP;    /* Enable interrupt on Stop, to catch it */
                    TempI2C_GENERATE_STOP;

                    /* Set short transfer and error flag */
                    TempI2C_mstrStatus |= (TempI2C_MSTAT_ERR_SHORT_XFER |
                                                    TempI2C_MSTAT_ERR_XFER);
                }

                break;

            case TempI2C_SM_MSTR_RD_DATA:

                TempI2C_mstrRdBufPtr[TempI2C_mstrRdBufIndex] = TempI2C_DATA_REG;
                TempI2C_mstrRdBufIndex++;

                /* Check if end of buffer */
                if(TempI2C_mstrRdBufIndex < TempI2C_mstrRdBufSize)
                {
                    TempI2C_ACK_AND_RECEIVE;       /* ACK and receive byte */
                }
                /* End of buffer: complete reading */
                else if(TempI2C_CHECK_NO_STOP(TempI2C_mstrControl))
                {
                    /* Set read complete and master halted */
                    TempI2C_mstrStatus |= (TempI2C_MSTAT_XFER_HALT |
                                                    TempI2C_MSTAT_RD_CMPLT);

                    TempI2C_state = TempI2C_SM_MSTR_HALT;    /* Expect ReStart */
                    TempI2C_DisableInt();
                }
                else
                {
                    TempI2C_ENABLE_INT_ON_STOP;
                    TempI2C_NAK_AND_RECEIVE;       /* NACK and TRY to generate Stop */
                }
                break;

            default: /* This is an invalid state and should not occur */

            #if(TempI2C_TIMEOUT_ENABLED)
                /* Exit interrupt to take chance for timeout timer to handles this case */
                TempI2C_DisableInt();
                TempI2C_ClearPendingInt();
            #else
                /* Block execution flow: unexpected condition */
                CYASSERT(0u != 0u);
            #endif /* (TempI2C_TIMEOUT_ENABLED) */

                break;
            }
        }

        /* Catches Stop: end of transaction */
        if(TempI2C_CHECK_STOP_STS(tmpCsr))
        {
            TempI2C_mstrStatus |= TempI2C_GET_MSTAT_CMPLT;

            TempI2C_DISABLE_INT_ON_STOP;
            TempI2C_state = TempI2C_SM_IDLE;
        }
    #endif /* (TempI2C_MODE_MASTER_ENABLED) */
    }
    else if(TempI2C_CHECK_SM_SLAVE)
    {
    #if(TempI2C_MODE_SLAVE_ENABLED)

        if((TempI2C_CHECK_STOP_STS(tmpCsr)) || /* Stop || Restart */
           (TempI2C_CHECK_BYTE_COMPLETE(tmpCsr) && TempI2C_CHECK_ADDRESS_STS(tmpCsr)))
        {
            /* Catch end of master write transaction: use interrupt on Stop */
            /* The Stop bit history on address phase does not have correct state */
            if(TempI2C_SM_SL_WR_DATA == TempI2C_state)
            {
                TempI2C_DISABLE_INT_ON_STOP;

                TempI2C_slStatus &= ((uint8) ~TempI2C_SSTAT_WR_BUSY);
                TempI2C_slStatus |= ((uint8)  TempI2C_SSTAT_WR_CMPLT);

                TempI2C_state = TempI2C_SM_IDLE;
            }
        }

        if(TempI2C_CHECK_BYTE_COMPLETE(tmpCsr))
        {
            /* The address only issued after Start or ReStart: so check the address
               to catch these events:
                FF : sets an address phase with a byte_complete interrupt trigger.
                UDB: sets an address phase immediately after Start or ReStart. */
            if(TempI2C_CHECK_ADDRESS_STS(tmpCsr))
            {
            /* Check for software address detection */
            #if(TempI2C_SW_ADRR_DECODE)
                tmp8 = TempI2C_GET_SLAVE_ADDR(TempI2C_DATA_REG);

                if(tmp8 == TempI2C_slAddress)   /* Check for address match */
                {
                    if(0u != (TempI2C_DATA_REG & TempI2C_READ_FLAG))
                    {
                        /* Place code to prepare read buffer here                  */
                        /* `#START TempI2C_SW_PREPARE_READ_BUF_interrupt` */

                        /* `#END` */

                    #ifdef TempI2C_SW_PREPARE_READ_BUF_CALLBACK
                        TempI2C_SwPrepareReadBuf_Callback();
                    #endif /* TempI2C_SW_PREPARE_READ_BUF_CALLBACK */
                        
                        /* Prepare next operation to read, get data and place in data register */
                        if(TempI2C_slRdBufIndex < TempI2C_slRdBufSize)
                        {
                            /* Load first data byte from array */
                            TempI2C_DATA_REG = TempI2C_slRdBufPtr[TempI2C_slRdBufIndex];
                            TempI2C_ACK_AND_TRANSMIT;
                            TempI2C_slRdBufIndex++;

                            TempI2C_slStatus |= TempI2C_SSTAT_RD_BUSY;
                        }
                        else    /* Overflow: provide 0xFF on bus */
                        {
                            TempI2C_DATA_REG = TempI2C_OVERFLOW_RETURN;
                            TempI2C_ACK_AND_TRANSMIT;

                            TempI2C_slStatus  |= (TempI2C_SSTAT_RD_BUSY |
                                                           TempI2C_SSTAT_RD_ERR_OVFL);
                        }

                        TempI2C_state = TempI2C_SM_SL_RD_DATA;
                    }
                    else  /* Write transaction: receive 1st byte */
                    {
                        TempI2C_ACK_AND_RECEIVE;
                        TempI2C_state = TempI2C_SM_SL_WR_DATA;

                        TempI2C_slStatus |= TempI2C_SSTAT_WR_BUSY;
                        TempI2C_ENABLE_INT_ON_STOP;
                    }
                }
                else
                {
                    /*     Place code to compare for additional address here    */
                    /* `#START TempI2C_SW_ADDR_COMPARE_interruptStart` */

                    /* `#END` */

                #ifdef TempI2C_SW_ADDR_COMPARE_ENTRY_CALLBACK
                    TempI2C_SwAddrCompare_EntryCallback();
                #endif /* TempI2C_SW_ADDR_COMPARE_ENTRY_CALLBACK */
                    
                    TempI2C_NAK_AND_RECEIVE;   /* NACK address */

                    /* Place code to end of condition for NACK generation here */
                    /* `#START TempI2C_SW_ADDR_COMPARE_interruptEnd`  */

                    /* `#END` */

                #ifdef TempI2C_SW_ADDR_COMPARE_EXIT_CALLBACK
                    TempI2C_SwAddrCompare_ExitCallback();
                #endif /* TempI2C_SW_ADDR_COMPARE_EXIT_CALLBACK */
                }

            #else /* (TempI2C_HW_ADRR_DECODE) */

                if(0u != (TempI2C_DATA_REG & TempI2C_READ_FLAG))
                {
                    /* Place code to prepare read buffer here                  */
                    /* `#START TempI2C_HW_PREPARE_READ_BUF_interrupt` */

                    /* `#END` */
                    
                #ifdef TempI2C_HW_PREPARE_READ_BUF_CALLBACK
                    TempI2C_HwPrepareReadBuf_Callback();
                #endif /* TempI2C_HW_PREPARE_READ_BUF_CALLBACK */

                    /* Prepare next operation to read, get data and place in data register */
                    if(TempI2C_slRdBufIndex < TempI2C_slRdBufSize)
                    {
                        /* Load first data byte from array */
                        TempI2C_DATA_REG = TempI2C_slRdBufPtr[TempI2C_slRdBufIndex];
                        TempI2C_ACK_AND_TRANSMIT;
                        TempI2C_slRdBufIndex++;

                        TempI2C_slStatus |= TempI2C_SSTAT_RD_BUSY;
                    }
                    else    /* Overflow: provide 0xFF on bus */
                    {
                        TempI2C_DATA_REG = TempI2C_OVERFLOW_RETURN;
                        TempI2C_ACK_AND_TRANSMIT;

                        TempI2C_slStatus  |= (TempI2C_SSTAT_RD_BUSY |
                                                       TempI2C_SSTAT_RD_ERR_OVFL);
                    }

                    TempI2C_state = TempI2C_SM_SL_RD_DATA;
                }
                else  /* Write transaction: receive 1st byte */
                {
                    TempI2C_ACK_AND_RECEIVE;
                    TempI2C_state = TempI2C_SM_SL_WR_DATA;

                    TempI2C_slStatus |= TempI2C_SSTAT_WR_BUSY;
                    TempI2C_ENABLE_INT_ON_STOP;
                }

            #endif /* (TempI2C_SW_ADRR_DECODE) */
            }
            /* Data states */
            /* Data master writes into slave */
            else if(TempI2C_state == TempI2C_SM_SL_WR_DATA)
            {
                if(TempI2C_slWrBufIndex < TempI2C_slWrBufSize)
                {
                    tmp8 = TempI2C_DATA_REG;
                    TempI2C_ACK_AND_RECEIVE;
                    TempI2C_slWrBufPtr[TempI2C_slWrBufIndex] = tmp8;
                    TempI2C_slWrBufIndex++;
                }
                else  /* of array: complete write, send NACK */
                {
                    TempI2C_NAK_AND_RECEIVE;

                    TempI2C_slStatus |= TempI2C_SSTAT_WR_ERR_OVFL;
                }
            }
            /* Data master reads from slave */
            else if(TempI2C_state == TempI2C_SM_SL_RD_DATA)
            {
                if(TempI2C_CHECK_DATA_ACK(tmpCsr))
                {
                    if(TempI2C_slRdBufIndex < TempI2C_slRdBufSize)
                    {
                         /* Get data from array */
                        TempI2C_DATA_REG = TempI2C_slRdBufPtr[TempI2C_slRdBufIndex];
                        TempI2C_TRANSMIT_DATA;
                        TempI2C_slRdBufIndex++;
                    }
                    else   /* Overflow: provide 0xFF on bus */
                    {
                        TempI2C_DATA_REG = TempI2C_OVERFLOW_RETURN;
                        TempI2C_TRANSMIT_DATA;

                        TempI2C_slStatus |= TempI2C_SSTAT_RD_ERR_OVFL;
                    }
                }
                else  /* Last byte was NACKed: read complete */
                {
                    /* Only NACK appears on bus */
                    TempI2C_DATA_REG = TempI2C_OVERFLOW_RETURN;
                    TempI2C_NAK_AND_TRANSMIT;

                    TempI2C_slStatus &= ((uint8) ~TempI2C_SSTAT_RD_BUSY);
                    TempI2C_slStatus |= ((uint8)  TempI2C_SSTAT_RD_CMPLT);

                    TempI2C_state = TempI2C_SM_IDLE;
                }
            }
            else
            {
            #if(TempI2C_TIMEOUT_ENABLED)
                /* Exit interrupt to take chance for timeout timer to handle this case */
                TempI2C_DisableInt();
                TempI2C_ClearPendingInt();
            #else
                /* Block execution flow: unexpected condition */
                CYASSERT(0u != 0u);
            #endif /* (TempI2C_TIMEOUT_ENABLED) */
            }
        }
    #endif /* (TempI2C_MODE_SLAVE_ENABLED) */
    }
    else
    {
        /* The FSM skips master and slave processing: return to IDLE */
        TempI2C_state = TempI2C_SM_IDLE;
    }

#ifdef TempI2C_ISR_EXIT_CALLBACK
    TempI2C_ISR_ExitCallback();
#endif /* TempI2C_ISR_EXIT_CALLBACK */    
}


#if ((TempI2C_FF_IMPLEMENTED) && (TempI2C_WAKEUP_ENABLED))
    /*******************************************************************************
    * Function Name: TempI2C_WAKEUP_ISR
    ********************************************************************************
    *
    * Summary:
    *  The interrupt handler to trigger after a wakeup.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    CY_ISR(TempI2C_WAKEUP_ISR)
    {
    #ifdef TempI2C_WAKEUP_ISR_ENTRY_CALLBACK
        TempI2C_WAKEUP_ISR_EntryCallback();
    #endif /* TempI2C_WAKEUP_ISR_ENTRY_CALLBACK */
         
        /* Set flag to notify that matched address is received */
        TempI2C_wakeupSource = 1u;

        /* SCL is stretched until the I2C_Wake() is called */

    #ifdef TempI2C_WAKEUP_ISR_EXIT_CALLBACK
        TempI2C_WAKEUP_ISR_ExitCallback();
    #endif /* TempI2C_WAKEUP_ISR_EXIT_CALLBACK */
    }
#endif /* ((TempI2C_FF_IMPLEMENTED) && (TempI2C_WAKEUP_ENABLED)) */


/* [] END OF FILE */
