/*******************************************************************************
* File Name: TempMonSMBusSlave_cmd.c
* Version 5.20
*
* Description:
*  This file provides the source code related to SM/PM bus command handling.
*
*******************************************************************************
* Copyright 2012-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "TempMonSMBusSlave_PVT.h"
#include "cyapicallbacks.h"

/* I2C Buffer */
uint8   TempMonSMBusSlave_buffer[TempMonSMBusSlave_MAX_BUFFER_SIZE];
uint8   TempMonSMBusSlave_bufferIndex;
uint8   TempMonSMBusSlave_bufferSize;

static uint8   TempMonSMBusSlave_cmdPage = 0u;
static uint8 * TempMonSMBusSlave_cmdDataPtr;
static uint8   TempMonSMBusSlave_cmdProperties;
static uint8   TempMonSMBusSlave_lastReceivedCmd;

static void    TempMonSMBusSlave_FillTransaction(uint8 command) ;

static const TempMonSMBusSlave_CMD_TABLE_ENTRY CYCODE TempMonSMBusSlave_commands[] = {
};


/*******************************************************************************
* Function Name: TempMonSMBusSlave_CheckCommand
********************************************************************************
*
* Summary:
*  Looks for a command in the array of supported commands.
*
* Parameters:
*  uint8 command: received command.
*
* Return:
*  TempMonSMBusSlave_ERR_UNSUPPORTED_CMD - command not found.
*  TempMonSMBusSlave_CMD_VALID - command found.
*
*******************************************************************************/
uint8 TempMonSMBusSlave_CheckCommand(uint8 command) 
{
    uint8 cmdIdx;

    /* Look for command in array of supported commands */
    for(cmdIdx = 0u; cmdIdx < TempMonSMBusSlave_NUM_COMMANDS; ++cmdIdx)
    {
        if(command == TempMonSMBusSlave_commands[cmdIdx].command)
        {
            /* Set write pointer to buffer start */
            TempMonSMBusSlave_bufferSize      = TempMonSMBusSlave_commands[cmdIdx].dataLength;
            TempMonSMBusSlave_lastReceivedCmd = command;
            TempMonSMBusSlave_cmdProperties   = TempMonSMBusSlave_commands[cmdIdx].cmdProp;
            TempMonSMBusSlave_cmdDataPtr      = TempMonSMBusSlave_commands[cmdIdx].dataPtr;

            /* Local copy of page command from the operating register store */
            #if(0u != TempMonSMBusSlave_SUPPORT_PAGE_CMD)
                TempMonSMBusSlave_cmdPage = TempMonSMBusSlave_regs.PAGE;
            #endif /* 0u != TempMonSMBusSlave_SUPPORT_PAGE_CMD */

            /* If PAGE is set to 0xFF, following transactions are treated as errors.
            *    - An attempt to Read from a Paged command
            *    - An attempt to Write to a Paged command that is configured as Auto
            */         
            if(0u != (TempMonSMBusSlave_cmdProperties & TempMonSMBusSlave_CMD_PAGED))
            {
                if(TempMonSMBusSlave_CMD_ALL_PAGES != TempMonSMBusSlave_cmdPage)
                {
                    /* Set data pointer to current page */
                    #if(0u != TempMonSMBusSlave_SUPPORT_PAGE_CMD)
                        TempMonSMBusSlave_cmdDataPtr += (uint16)TempMonSMBusSlave_bufferSize * TempMonSMBusSlave_cmdPage;
                    #endif /* 0u != TempMonSMBusSlave_SUPPORT_PAGE_CMD */
                }
                else if(0u == (TempMonSMBusSlave_cmdProperties & TempMonSMBusSlave_CMD_WR_AUTO))
                {
                    TempMonSMBusSlave_cmdProperties &= (TempMonSMBusSlave_CMD_PAGED | TempMonSMBusSlave_CMD_WR_MANUAL);
                }
                else
                {
                    TempMonSMBusSlave_cmdProperties = 0u;
                }
            }

            break; /* Command found - no need to continue */
        }
    }

    return ((TempMonSMBusSlave_NUM_COMMANDS == cmdIdx) ? TempMonSMBusSlave_ERR_UNSUPPORTED_CMD : TempMonSMBusSlave_CMD_VALID);
}


/*******************************************************************************
* Function Name: TempMonSMBusSlave_CompleteRead
********************************************************************************
*
* Summary:
*  This is a handler for commands that have Read configuration set to "Auto" or
*  "Manual". It copies data from Operating Memory (RAM) into the I2C buffer.
*  For an "Auto" transaction it is called before an address acknowledge at
*  ReStart. For "Manual" it is in the same phase but after
*  TempMonSMBusSlave_ReadManualHandler.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void TempMonSMBusSlave_CompleteRead(void) 
{
    switch(TempMonSMBusSlave_lastReceivedCmd)
    {
        

        default:
            break;
    }

    /* For PSoC 4 enable interrupt for TX FIFO empty source. On PSoC 3/5LP
    * first data byte has to be loaded on address phase.
    */
    #if(CY_PSOC4)
        TempMonSMBusSlave_I2C_SetTxInterruptMode(TempMonSMBusSlave_I2C_INTR_TX_EMPTY);
    #else
        TempMonSMBusSlave_I2C_DATA_REG = TempMonSMBusSlave_buffer[0u];
        TempMonSMBusSlave_I2C_ACK_AND_TRANSMIT;
    #endif /* !CY_PSOC4 */
}


/*******************************************************************************
* Function Name: TempMonSMBusSlave_WriteHandler
********************************************************************************
*
* Summary:
*  Handles data processing for a "Write" part of SM/PM Bus transaction.
*
* Parameters:
*  None
*
* Return:
*  uint8 status:
*   Options: 0=success, error code on fault
*
*******************************************************************************/
void TempMonSMBusSlave_WriteHandler(void) 
{
    if(0u != (TempMonSMBusSlave_cmdProperties & TempMonSMBusSlave_CMD_WR_AUTO))
    {
        switch(TempMonSMBusSlave_lastReceivedCmd)
        {
            

            default:
                break;
        }
    }
    
    /* `#START ADDITIONAL_USER_CODE_FOR_BOOTLOADER` */

    /* `#END` */
    
    #ifdef TempMonSMBusSlave_WRITE_HANDLER_CALLBACK
        TempMonSMBusSlave_WriteHandler_Callback();
    #endif /* TempMonSMBusSlave_WRITE_HANDLER_CALLBACK */
    
    else if (0u != (TempMonSMBusSlave_cmdProperties & TempMonSMBusSlave_CMD_WR_MANUAL))
    {
        if(TempMonSMBusSlave_CMD_LEN_VALID)
        {
            TempMonSMBusSlave_FillTransaction(TempMonSMBusSlave_CMD_WRITE);
        }
    }
    else /* Data Content Fault. Ignore received data */
    {
        TempMonSMBusSlave_HandleBusError(TempMonSMBusSlave_ERR_INVALID_DATA);
    }
}


/*******************************************************************************
* Function Name: TempMonSMBusSlave_ReadHandler
********************************************************************************
*
* Summary:
*  Handles data processing for a "Read" part of SM/PM Bus transaction.
*
* Parameters:
*  None
*
* Return:
*  uint8 status:
*   Options: 0=success, error code on fault
*
*******************************************************************************/
uint8 TempMonSMBusSlave_ReadHandler(void) 
{
    uint8 error = 0u;

    if(0u != (TempMonSMBusSlave_cmdProperties & TempMonSMBusSlave_CMD_RD_AUTO))
    {  
        TempMonSMBusSlave_CompleteRead();
    }
    else if(0u != (TempMonSMBusSlave_cmdProperties & TempMonSMBusSlave_CMD_RD_MANUAL))
    {
        TempMonSMBusSlave_FillTransaction(TempMonSMBusSlave_CMD_READ);
    }
    else /* Data Content Fault. FFh will be sent as host keeps clocking */
    {
        error = TempMonSMBusSlave_ERR_INVALID_DATA;
    }
    return (error);
}


/*******************************************************************************
* Function Name: TempMonSMBusSlave_FillTransaction
********************************************************************************
*
* Summary:
*  Fills a transaction queue with all the user required data for "Manual"
*  handling of SM/PM Bus transaction.
*
* Parameters:
*  uint8 command: command type (read or write)
*
* Return:
*  None
*
*******************************************************************************/
static void TempMonSMBusSlave_FillTransaction(uint8 command) 
{    
    /* Copy all data from last transaction into transaction struct */
    TempMonSMBusSlave_transactionData[0u].read        = command;
    TempMonSMBusSlave_transactionData[0u].commandCode = TempMonSMBusSlave_lastReceivedCmd;
    TempMonSMBusSlave_transactionData[0u].length      = TempMonSMBusSlave_bufferSize;

    /* Fill page information */
    if (0u != (TempMonSMBusSlave_cmdProperties & TempMonSMBusSlave_CMD_PAGED))
    {
        TempMonSMBusSlave_transactionData[0u].page = TempMonSMBusSlave_cmdPage;
    }
    else
    {
        /* For non-paged commands page number is zero */
        TempMonSMBusSlave_transactionData[0u].page = 0u;
    }
    
    /* For Write and Process Call commands copy all data to payload */
    while(0u != TempMonSMBusSlave_bufferIndex)
    {
        /* Copy all data into transaction structure */
        --TempMonSMBusSlave_bufferIndex;
        TempMonSMBusSlave_transactionData[0u].payload[TempMonSMBusSlave_bufferIndex] = TempMonSMBusSlave_buffer[TempMonSMBusSlave_bufferIndex];
    }
    
    TempMonSMBusSlave_trActiveCount = 1u;
    
    /* Disable interrupt and wait for TempMonSMBusSlave_CompleteTransaction() */
    TempMonSMBusSlave_I2C_DisableInt();
}


/* [] END OF FILE */
