/*******************************************************************************
* File Name: TempSMBus_INT.c
* Version 5.20
*
* Description:
*  This file provides the source code of the Interrupt Service Routine (ISR)
*  for the SM/PMBus component.
*
********************************************************************************
* Copyright 2012-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "TempSMBus_PVT.h"
#include "cyapicallbacks.h"

/* User section to add header files and variable declarations */
/*`#START VAR_INCLUDE_REGION`*/

/*`#END`*/

static uint8 TempSMBus_isCmdReceived = 0u;
static uint8 TempSMBus_error = 0u;
        
        
/*******************************************************************************
* Function Name: TempSMBus_ISR
********************************************************************************
*
* Summary:
*  The Interrupt Service Routine for the SM/PMBus Interrupt.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Reentrant:
*  No
*
*******************************************************************************/
CY_ISR(TempSMBus_ISR)
{
    /* Computed CRC value (after the process runs). For each byte received in a 
    * packet a new CRC is computed. For the first byte in a packet the CRC is
    * reset. The last byte in a packet is a CRC  (PEC) for the packet. The 
    * received PEC is compared with a computed CRC to verify the correctness of
	* the PEC.
    */
    #if (TempSMBus_SUPPORT_PEC)
        static uint8 TempSMBus_crc_value = 0u;
    #endif /* TempSMBus_SUPPORT_PEC */

    uint8 tmpCsr;   /* Local copy of I2C Control/Status Register */
    uint8 newByte;  /* Data byte received in given transaction */

    #ifdef TempSMBus_ISR_ENTRY_CALLBACK
        TempSMBus_ISR_EntryCallback();
    #endif /* TempSMBus_ISR_ENTRY_CALLBACK */

    /* Add your custom code between the following #START and #END tags */
    /* `#START ISR_ENTRY_REGION` */

    /* `#END` */
    
    tmpCsr = TempSMBus_I2C_CSR_REG;   /* Make copy as interrupts clear */

    /* Complete write transaction and execute command if Stop bit was received */
    if(TempSMBus_I2C_CHECK_STOP_STS(tmpCsr))
    {
        /* The component detects any Stop condition on the bus. Therefore an
        * additional check of the internal FSM state is required to determine
        * whether the host finishes the transaction to the component.
        */
        if(TempSMBus_I2C_SM_SL_WR_DATA == TempSMBus_I2C_state)
        {
            TempSMBus_I2C_DISABLE_INT_ON_STOP;
            TempSMBus_I2C_state = TempSMBus_I2C_SM_IDLE;
            
            /* Process and execute command */
            TempSMBus_isCmdReceived = 0u;
            if(0u == TempSMBus_error)
            {
                TempSMBus_WriteHandler();
            }
            else
            {
                TempSMBus_HandleBusError(TempSMBus_error);
            }
        }
    }

    if(TempSMBus_I2C_CHECK_BYTE_COMPLETE(tmpCsr))
    {
        if(TempSMBus_I2C_CHECK_ADDRESS_STS(tmpCsr))
        {            
            newByte = TempSMBus_I2C_DATA_REG;

        #if (TempSMBus_SW_ADDR_DECODE)
            if(((newByte & TempSMBus_ADDR_MASK) == TempSMBus_address) ||
                (newByte == TempSMBus_GENERAL_CALL_ADDR))
        #endif /* TempSMBus_SW_ADDR_DECODE */
            {
                /* Add your custom code between the following #START and #END tags */
                /* `#START PRIMARY_ADDRESS_MATCH_REGION` */

                /* `#END` */
                
                #ifdef TempSMBus_PRIMARY_ADDRESS_MATCH_CALLBACK
                    TempSMBus_ISR_PrimaryAddressMatch_Callback();
                #endif /* TempSMBus_PRIMARY_ADDRESS_MATCH_CALLBACK */
                
                /* Check for read or write transaction */
                if(0u != (newByte & TempSMBus_I2C_READ_FLAG))
                {
                    if(0u != TempSMBus_isCmdReceived)
                    {
                        /* Determine if the received command is supported */
                        if(0u == TempSMBus_error)
                        {
                            TempSMBus_error = TempSMBus_ReadHandler();
                            #if (TempSMBus_SUPPORT_PEC)
                                newByte ^= TempSMBus_crc_value;
                            #endif /* TempSMBus_SUPPORT_PEC */
                        }
                    }
                    else
                    {
                        #if(TempSMBus_SMBUS_MODE && TempSMBus_RECEIVE_BYTE_PROT_ENABLED)

                            /* If command is not received, service the receive byte protocol.
                            * Call user defined API to provide data byte for this transaction.
                            */
                            TempSMBus_buffer[0u] = TempSMBus_GetReceiveByteResponse();
                            TempSMBus_I2C_DATA_REG = TempSMBus_buffer[0u];
                            TempSMBus_I2C_ACK_AND_TRANSMIT;
                            TempSMBus_bufferSize = 1u;
                        #else
                            TempSMBus_error = TempSMBus_ERR_READ_FLAG;
                        #endif /*TempSMBus_SMBUS_MODE && TempSMBus_RECEIVE_BYTE_PROT_ENABLED */
                    }
                    if(0u != TempSMBus_error)
                    {
                        /* Data Content Fault. Send FFh as host keeps clocking. */
                        TempSMBus_I2C_DATA_REG = 0xFFu;
                        TempSMBus_I2C_ACK_AND_TRANSMIT;
                    }
                    TempSMBus_I2C_state = TempSMBus_I2C_SM_SL_RD_DATA;
                }
                else  /* Start of Write transaction, ready to write first byte */
                { 
                    TempSMBus_I2C_ENABLE_INT_ON_STOP;
                    TempSMBus_I2C_ACK_AND_RECEIVE;
                    TempSMBus_I2C_state = TempSMBus_I2C_SM_SL_WR_DATA;
                }
                #if (TempSMBus_SUPPORT_PEC)
                    TempSMBus_crc_value = TempSMBus_crc_table[newByte];
                #endif /* TempSMBus_SUPPORT_PEC */

                TempSMBus_bufferIndex = 0u; /* Reset buffer index */
            }
        #if(TempSMBus_SMBALERT_PIN_EXISTS)
            /* When SMBALERT# pin is exposed in the component, it is required
            * to monitor if the master is broadcasting the Alert Response Address.
            * If ARA was detected and SMBALERT# pin is asserted by the device,
            * respond with the device address and handle SMBALERT# pin.
            */
            else if((0u == TempSMBus_SMBALERT_PIN_REG) && (newByte == TempSMBus_alertResponseAddress))
            {
                #if (TempSMBus_SUPPORT_PEC)
                    TempSMBus_crc_value  = TempSMBus_crc_table[newByte];
                #endif /* TempSMBus_SUPPORT_PEC */
            
                TempSMBus_I2C_DATA_REG = TempSMBus_address;
                TempSMBus_I2C_ACK_AND_TRANSMIT;
                TempSMBus_buffer[0u] = TempSMBus_address;
                TempSMBus_bufferSize = 1u;

                switch(TempSMBus_smbAlertMode)
                {
                case TempSMBus_AUTO_MODE:  
                    TempSMBus_SetSmbAlert(1u); /* De-assert SMBALERT# pin (active low) */
                    break;
                case TempSMBus_MANUAL_MODE:
                    TempSMBus_HandleSmbAlertResponse(); /* Call user provided API to handle SMBALERT# pin */
                    break;
                default:
                    break;  /* DO_NOTHING mode: component takes no action */
                }
                
                /* Move to Read Data state to send last byte (0xFF) */
                TempSMBus_I2C_state = TempSMBus_I2C_SM_SL_RD_DATA;
                TempSMBus_bufferIndex = 0u;  /* Reset buffer index */
            }
        #endif /* TempSMBus_SMBALERT_PIN_EXISTS */
            
        #if (TempSMBus_SW_ADDR_DECODE)
            else /* No address match */
            {
                TempSMBus_I2C_NAK_AND_RECEIVE;   /* NACK received address */
            }
        #endif /* TempSMBus_SW_ADDR_DECODE */

        }

        /* Write transaction from master to slave */
        else if(TempSMBus_I2C_SM_SL_WR_DATA == TempSMBus_I2C_state)
        {
             newByte = TempSMBus_I2C_DATA_REG;

            /* Check if command code was received and supported */
            if(0u == TempSMBus_isCmdReceived)
            {
                TempSMBus_error = TempSMBus_CheckCommand(newByte);
                TempSMBus_isCmdReceived = 1u;
            }
            /* Data portion of write transaction */
            else
            {
                if(TempSMBus_bufferIndex < TempSMBus_bufferSize)
                {
                    TempSMBus_buffer[TempSMBus_bufferIndex] = newByte;
                }
            #if (TempSMBus_SUPPORT_PEC)
                else if(TempSMBus_bufferIndex == TempSMBus_bufferSize)
                {
                    if(newByte != TempSMBus_crc_value)
                    {
                        TempSMBus_error = TempSMBus_ERR_CORRUPTED_DATA;
                    }
                }
            #endif /* TempSMBus_SUPPORT_PEC */
                else /* Data Transmission Fault: Host Sends Too Many Bytes */
                {
                    TempSMBus_error = TempSMBus_ERR_WR_TO_MANY_BYTES;
                }
                ++TempSMBus_bufferIndex;
            }

            if(0u == TempSMBus_error)
            {
                TempSMBus_I2C_ACK_AND_RECEIVE;
                #if (TempSMBus_SUPPORT_PEC)
                    TempSMBus_crc_value ^= newByte;
                    TempSMBus_crc_value = TempSMBus_crc_table[TempSMBus_crc_value];
                #endif /* TempSMBus_SUPPORT_PEC */
            }
            else /* NACK received byte */
            {
                TempSMBus_I2C_NAK_AND_RECEIVE;
            }
        }
        /* Read transaction from slave to master */
        else if(TempSMBus_I2C_SM_SL_RD_DATA == TempSMBus_I2C_state)
        {
            if(TempSMBus_I2C_CHECK_DATA_ACK(tmpCsr))
            {
                if(0u == TempSMBus_error)
                {
                    #if (TempSMBus_SUPPORT_PEC)
                        TempSMBus_crc_value ^= TempSMBus_buffer[TempSMBus_bufferIndex];
                        TempSMBus_crc_value = TempSMBus_crc_table[TempSMBus_crc_value];
                    #endif /* TempSMBus_SUPPORT_PEC */
                    
                    /* First byte is sent in address state. Increment index to next byte. */
                    ++TempSMBus_bufferIndex;
                    if(TempSMBus_bufferIndex < TempSMBus_bufferSize)
                    {
                        TempSMBus_I2C_DATA_REG = TempSMBus_buffer[TempSMBus_bufferIndex];
                    }
                #if (TempSMBus_SUPPORT_PEC)
                    else if(TempSMBus_bufferIndex == TempSMBus_bufferSize)
                    {
                        /* Last data byte was sent. Supply PEC. */
                        TempSMBus_I2C_DATA_REG = TempSMBus_crc_value;
                    }
                #endif /* TempSMBus_SUPPORT_PEC */
                    else /* Master attempts to read to many bytes. Send 0xFF and ACK */
                    {
                        TempSMBus_I2C_DATA_REG = 0xFFu;
                        TempSMBus_error = TempSMBus_ERR_RD_TO_MANY_BYTES;
                    }
                }
                else /* Data Content Fault. Send 0xFF and ACK */
                {
                    TempSMBus_I2C_DATA_REG = 0xFFu;
                }
                TempSMBus_I2C_TRANSMIT_DATA;
            }
            else  /* Last byte NACKed, done */
            {                
                TempSMBus_I2C_DATA_REG = 0xFFu;
                TempSMBus_I2C_NAK_AND_TRANSMIT;
                TempSMBus_I2C_DISABLE_INT_ON_STOP;
                TempSMBus_I2C_state = TempSMBus_I2C_SM_IDLE;
                
                #ifdef TempSMBus_BTLDR_READ_EN
                    TempSMBus_btldrStatus |= TempSMBus_BTLDR_RD_CMPT;
                #endif /* TempSMBus_BTLDR_READ_EN */
                
                TempSMBus_isCmdReceived = 0u;
                if(TempSMBus_error != 0u)
                {
                    TempSMBus_HandleBusError(TempSMBus_error);
                    TempSMBus_error = 0u;
                }
            }
        }
        else /* This is invalid state and should not occur */
        {
            TempSMBus_I2C_NAK_AND_RECEIVE;
        }
    }
    
    #ifdef TempSMBus_ISR_EXIT_CALLBACK
        TempSMBus_ISR_ExitCallback();
    #endif /* TempSMBus_ISR_EXIT_CALLBACK */
}


/*******************************************************************************
* Function Name: TempSMBus_TIMEOUT_ISR
********************************************************************************
*
* Summary:
*  Handles a bus timeout occurrence event.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Reentrant:
*  No
*
*******************************************************************************/
CY_ISR(TempSMBus_TIMEOUT_ISR)
{
    #ifdef TempSMBus_TIMEOUT_ISR_ENTRY_CALLBACK
        TempSMBus_TIMEOUT_ISR_EntryCallback();
    #endif /* TempSMBus_TIMEOUT_ISR_ENTRY_CALLBACK */
    
    TempSMBus_I2C_Stop();
    
    /* Inform user that timeout occurred */
    TempSMBus_HandleBusError(TempSMBus_ERR_TIMEOUT);
    
    /* Reset all variables */
    TempSMBus_trActiveCount = 0u;
    TempSMBus_isCmdReceived = 0u;
    TempSMBus_error = 0u;
    TempSMBus_I2C_Enable();
    TempSMBus_EnableInt();

    #ifdef TempSMBus_TIMEOUT_ISR_EXIT_CALLBACK
        TempSMBus_TIMEOUT_ISR_ExitCallback();
    #endif /* TempSMBus_TIMEOUT_ISR_EXIT_CALLBACK */
}


/* [] END OF FILE */
