/*******************************************************************************
* File Name: TempSMBus_cmd.c
* Version 5.20
*
* Description:
*  This file provides the source code related to SM/PM bus command handling.
*
*******************************************************************************
* Copyright 2012-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "TempSMBus_PVT.h"
#include "cyapicallbacks.h"

/* I2C Buffer */
uint8   TempSMBus_buffer[TempSMBus_MAX_BUFFER_SIZE];
uint8   TempSMBus_bufferIndex;
uint8   TempSMBus_bufferSize;

static uint8   TempSMBus_cmdPage = 0u;
static uint8 * TempSMBus_cmdDataPtr;
static uint8   TempSMBus_cmdProperties;
static uint8   TempSMBus_lastReceivedCmd;

static void    TempSMBus_FillTransaction(uint8 command) ;

static const TempSMBus_CMD_TABLE_ENTRY CYCODE TempSMBus_commands[] = {
};


/*******************************************************************************
* Function Name: TempSMBus_CheckCommand
********************************************************************************
*
* Summary:
*  Looks for a command in the array of supported commands.
*
* Parameters:
*  uint8 command: received command.
*
* Return:
*  TempSMBus_ERR_UNSUPPORTED_CMD - command not found.
*  TempSMBus_CMD_VALID - command found.
*
*******************************************************************************/
uint8 TempSMBus_CheckCommand(uint8 command) 
{
    uint8 cmdIdx;

    /* Look for command in array of supported commands */
    for(cmdIdx = 0u; cmdIdx < TempSMBus_NUM_COMMANDS; ++cmdIdx)
    {
        if(command == TempSMBus_commands[cmdIdx].command)
        {
            /* Set write pointer to buffer start */
            TempSMBus_bufferSize      = TempSMBus_commands[cmdIdx].dataLength;
            TempSMBus_lastReceivedCmd = command;
            TempSMBus_cmdProperties   = TempSMBus_commands[cmdIdx].cmdProp;
            TempSMBus_cmdDataPtr      = TempSMBus_commands[cmdIdx].dataPtr;

            /* Local copy of page command from the operating register store */
            #if(0u != TempSMBus_SUPPORT_PAGE_CMD)
                TempSMBus_cmdPage = TempSMBus_regs.PAGE;
            #endif /* 0u != TempSMBus_SUPPORT_PAGE_CMD */

            /* If PAGE is set to 0xFF, following transactions are treated as errors.
            *    - An attempt to Read from a Paged command
            *    - An attempt to Write to a Paged command that is configured as Auto
            */         
            if(0u != (TempSMBus_cmdProperties & TempSMBus_CMD_PAGED))
            {
                if(TempSMBus_CMD_ALL_PAGES != TempSMBus_cmdPage)
                {
                    /* Set data pointer to current page */
                    #if(0u != TempSMBus_SUPPORT_PAGE_CMD)
                        TempSMBus_cmdDataPtr += (uint16)TempSMBus_bufferSize * TempSMBus_cmdPage;
                    #endif /* 0u != TempSMBus_SUPPORT_PAGE_CMD */
                }
                else if(0u == (TempSMBus_cmdProperties & TempSMBus_CMD_WR_AUTO))
                {
                    TempSMBus_cmdProperties &= (TempSMBus_CMD_PAGED | TempSMBus_CMD_WR_MANUAL);
                }
                else
                {
                    TempSMBus_cmdProperties = 0u;
                }
            }

            break; /* Command found - no need to continue */
        }
    }

    return ((TempSMBus_NUM_COMMANDS == cmdIdx) ? TempSMBus_ERR_UNSUPPORTED_CMD : TempSMBus_CMD_VALID);
}


/*******************************************************************************
* Function Name: TempSMBus_CompleteRead
********************************************************************************
*
* Summary:
*  This is a handler for commands that have Read configuration set to "Auto" or
*  "Manual". It copies data from Operating Memory (RAM) into the I2C buffer.
*  For an "Auto" transaction it is called before an address acknowledge at
*  ReStart. For "Manual" it is in the same phase but after
*  TempSMBus_ReadManualHandler.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void TempSMBus_CompleteRead(void) 
{
    switch(TempSMBus_lastReceivedCmd)
    {
        

        default:
            break;
    }

    /* For PSoC 4 enable interrupt for TX FIFO empty source. On PSoC 3/5LP
    * first data byte has to be loaded on address phase.
    */
    #if(CY_PSOC4)
        TempSMBus_I2C_SetTxInterruptMode(TempSMBus_I2C_INTR_TX_EMPTY);
    #else
        TempSMBus_I2C_DATA_REG = TempSMBus_buffer[0u];
        TempSMBus_I2C_ACK_AND_TRANSMIT;
    #endif /* !CY_PSOC4 */
}


/*******************************************************************************
* Function Name: TempSMBus_WriteHandler
********************************************************************************
*
* Summary:
*  Handles data processing for a "Write" part of SM/PM Bus transaction.
*
* Parameters:
*  None
*
* Return:
*  uint8 status:
*   Options: 0=success, error code on fault
*
*******************************************************************************/
void TempSMBus_WriteHandler(void) 
{
    if(0u != (TempSMBus_cmdProperties & TempSMBus_CMD_WR_AUTO))
    {
        switch(TempSMBus_lastReceivedCmd)
        {
            

            default:
                break;
        }
    }
    
    /* `#START ADDITIONAL_USER_CODE_FOR_BOOTLOADER` */

    /* `#END` */
    
    #ifdef TempSMBus_WRITE_HANDLER_CALLBACK
        TempSMBus_WriteHandler_Callback();
    #endif /* TempSMBus_WRITE_HANDLER_CALLBACK */
    
    else if (0u != (TempSMBus_cmdProperties & TempSMBus_CMD_WR_MANUAL))
    {
        if(TempSMBus_CMD_LEN_VALID)
        {
            TempSMBus_FillTransaction(TempSMBus_CMD_WRITE);
        }
    }
    else /* Data Content Fault. Ignore received data */
    {
        TempSMBus_HandleBusError(TempSMBus_ERR_INVALID_DATA);
    }
}


/*******************************************************************************
* Function Name: TempSMBus_ReadHandler
********************************************************************************
*
* Summary:
*  Handles data processing for a "Read" part of SM/PM Bus transaction.
*
* Parameters:
*  None
*
* Return:
*  uint8 status:
*   Options: 0=success, error code on fault
*
*******************************************************************************/
uint8 TempSMBus_ReadHandler(void) 
{
    uint8 error = 0u;

    if(0u != (TempSMBus_cmdProperties & TempSMBus_CMD_RD_AUTO))
    {  
        TempSMBus_CompleteRead();
    }
    else if(0u != (TempSMBus_cmdProperties & TempSMBus_CMD_RD_MANUAL))
    {
        TempSMBus_FillTransaction(TempSMBus_CMD_READ);
    }
    else /* Data Content Fault. FFh will be sent as host keeps clocking */
    {
        error = TempSMBus_ERR_INVALID_DATA;
    }
    return (error);
}


/*******************************************************************************
* Function Name: TempSMBus_FillTransaction
********************************************************************************
*
* Summary:
*  Fills a transaction queue with all the user required data for "Manual"
*  handling of SM/PM Bus transaction.
*
* Parameters:
*  uint8 command: command type (read or write)
*
* Return:
*  None
*
*******************************************************************************/
static void TempSMBus_FillTransaction(uint8 command) 
{    
    /* Copy all data from last transaction into transaction struct */
    TempSMBus_transactionData[0u].read        = command;
    TempSMBus_transactionData[0u].commandCode = TempSMBus_lastReceivedCmd;
    TempSMBus_transactionData[0u].length      = TempSMBus_bufferSize;

    /* Fill page information */
    if (0u != (TempSMBus_cmdProperties & TempSMBus_CMD_PAGED))
    {
        TempSMBus_transactionData[0u].page = TempSMBus_cmdPage;
    }
    else
    {
        /* For non-paged commands page number is zero */
        TempSMBus_transactionData[0u].page = 0u;
    }
    
    /* For Write and Process Call commands copy all data to payload */
    while(0u != TempSMBus_bufferIndex)
    {
        /* Copy all data into transaction structure */
        --TempSMBus_bufferIndex;
        TempSMBus_transactionData[0u].payload[TempSMBus_bufferIndex] = TempSMBus_buffer[TempSMBus_bufferIndex];
    }
    
    TempSMBus_trActiveCount = 1u;
    
    /* Disable interrupt and wait for TempSMBus_CompleteTransaction() */
    TempSMBus_I2C_DisableInt();
}


/* [] END OF FILE */
